from brian import *
from playdoh import *
import brian.optimiser as optimiser
#import pycuda.autoinit as autoinit
import pycuda.driver as drv
import pycuda
from pycuda.gpuarray import GPUArray
from pycuda import gpuarray
try:
    from pycuda.compiler import SourceModule
except ImportError:
    from pycuda.driver import SourceModule
from numpy import *
from brian.experimental.codegen.integration_schemes import *
from brian.experimental.codegen.codegen_gpu import *
import re

__all__ = ['GPUModelFitting',
           'euler_scheme', 'rk2_scheme', 'exp_euler_scheme'
           ]

if drv.get_version() == (2, 0, 0): # cuda version
    default_precision = 'float'
elif drv.get_version() > (2, 0, 0):
    default_precision = 'double'
else:
    raise Exception, "CUDA 2.0 required"

class ModelfittingGPUCodeGenerator(GPUCodeGenerator):
    def generate(self, eqs, scheme):
        vartype = self.vartype()
        code = ''
        for line in self.scheme(eqs, scheme).split('\n'):
            line = line.strip()
            if line:
                code += '    ' + line + '\n'
        return code

modelfitting_kernel_template = """
__global__ void runsim(
    int Tstart, int Tend,     // Start, end time as integer (t=T*dt)
    // State variables
    %SCALAR% *state_vars,     // State variables are offset from this
    double *I_arr,            // Input current
    int *I_arr_offset,        // Input current offset (for separate input
                              // currents for each neuron)
    int *spikecount,          // Number of spikes produced by each neuron
    int *num_coincidences,    // Count of coincidences for each neuron
    int *spiketimes,          // Array of all spike times as integers (begin and
                              // end each train with large negative value)
    int *spiketime_indices,   // Pointer into above array for each neuron
    int *spikedelay_arr,      // Integer delay for each spike
    int *refractory_arr,      // Integer refractory times
    int *next_allowed_spiketime_arr,// Integer time of the next allowed spike (for refractoriness)
    int onset                 // Time onset (only count spikes from here onwards)
    %COINCIDENCE_COUNT_DECLARE_EXTRA_STATE_VARIABLES%
    )
{
    const int neuron_index = blockIdx.x * blockDim.x + threadIdx.x;
    if(neuron_index>=%NUM_NEURONS%) return;
    %EXTRACT_STATE_VARIABLES%
    // Load variables at start
    %LOAD_VARIABLES%
    int spiketime_index = spiketime_indices[neuron_index];
    int last_spike_time = spiketimes[spiketime_index];
    int next_spike_time = spiketimes[spiketime_index+1];
    %COINCIDENCE_COUNT_INIT%
    int ncoinc = num_coincidences[neuron_index];
    int nspikes = spikecount[neuron_index];
    int I_offset = I_arr_offset[neuron_index];
    int spikedelay = spikedelay_arr[neuron_index];
    const int refractory = refractory_arr[neuron_index];
    int next_allowed_spiketime = next_allowed_spiketime_arr[neuron_index];
    for(int T=Tstart; T<Tend; T++)
    {
        %SCALAR% t = T*%DT%;
        // Read input current
        %SCALAR% I = I_arr[T+I_offset];
                             // this is a global read for each thread, can maybe
                             // reduce this by having just one read per block,
                             // put it into shared memory, and then have all the
                             // threads in that block read it, we could even
                             // maybe buffer reads of I into shared memory -
                             // experiment with this 
        // State update
        %STATE_UPDATE%
        // Threshold
        const bool is_refractory = (T<=next_allowed_spiketime);
        const bool has_spiked = (%THRESHOLD%)&&!is_refractory;
        nspikes += has_spiked*(T>=onset);
        // Reset
        if(has_spiked||is_refractory)
        {
            %RESET%
        }
        if(has_spiked)
            next_allowed_spiketime = T+refractory;
        // Coincidence counter
        const int Tspike = T+spikedelay;
        %COINCIDENCE_COUNT_TEST%
        if(Tspike>=next_spike_time){
            spiketime_index++;
            last_spike_time = next_spike_time;
            next_spike_time = spiketimes[spiketime_index+1];
            %COINCIDENCE_COUNT_NEXT%
        }
    }
    // Store variables at end
    %STORE_VARIABLES%
    %COINCIDENCE_COUNT_STORE_VARIABLES%
    next_allowed_spiketime_arr[neuron_index] = next_allowed_spiketime;
    spiketime_indices[neuron_index] = spiketime_index;
    num_coincidences[neuron_index] = ncoinc;
    spikecount[neuron_index] = nspikes;
}
"""

coincidence_counting_algorithm_src = {
    'inclusive':{
        '%COINCIDENCE_COUNT_DECLARE_EXTRA_STATE_VARIABLES%':'',
        '%COINCIDENCE_COUNT_INIT%':'',
        '%COINCIDENCE_COUNT_TEST%':'''
            ncoinc += has_spiked && (((last_spike_time+%DELTA%)>=Tspike) || ((next_spike_time-%DELTA%)<=Tspike));
            ''',
        '%COINCIDENCE_COUNT_NEXT%':'',
        '%COINCIDENCE_COUNT_STORE_VARIABLES%':'',
        },
    'exclusive':{
        '%COINCIDENCE_COUNT_DECLARE_EXTRA_STATE_VARIABLES%':''',
            bool *last_spike_allowed_arr,
            bool *next_spike_allowed_arr
        ''',
        '%COINCIDENCE_COUNT_INIT%':'''
            bool last_spike_allowed = last_spike_allowed_arr[neuron_index];
            bool next_spike_allowed = next_spike_allowed_arr[neuron_index];
            ''',
        '%COINCIDENCE_COUNT_TEST%':'''
            bool near_last_spike = last_spike_time+%DELTA%>=Tspike;
            bool near_next_spike = next_spike_time-%DELTA%<=Tspike;
            near_last_spike = near_last_spike && has_spiked;
            near_next_spike = near_next_spike && has_spiked;
            ncoinc += (near_last_spike&&last_spike_allowed) || (near_next_spike&&next_spike_allowed);
            bool near_both_allowed = (near_last_spike&&last_spike_allowed) && (near_next_spike&&next_spike_allowed);
            last_spike_allowed = last_spike_allowed && !near_last_spike;
            next_spike_allowed = (next_spike_allowed && !near_next_spike) || near_both_allowed;
            ''',
        '%COINCIDENCE_COUNT_NEXT%':'''
            last_spike_allowed = next_spike_allowed;
            next_spike_allowed = true;
            ''',
        '%COINCIDENCE_COUNT_STORE_VARIABLES%':'''
            last_spike_allowed_arr[neuron_index] = last_spike_allowed;
            next_spike_allowed_arr[neuron_index] = next_spike_allowed;
            ''',
        },
    }

def generate_modelfitting_kernel_src(G, eqs, threshold, reset, dt, num_neurons,
                                     delta,
                                     coincidence_count_algorithm='exclusive',
                                     precision=default_precision,
                                     scheme=euler_scheme
                                     ):
    eqs.prepare()
    src = modelfitting_kernel_template
    # Substitute state variable declarations
    indexvar = dict((v, k) for k, v in G.var_index.iteritems() if isinstance(k, str) and k!='I')
    extractions = '\n    '.join('%SCALAR% *'+name+'_arr = state_vars+'+str(i*num_neurons)+';' for i, name in indexvar.iteritems())
    src = src.replace('%EXTRACT_STATE_VARIABLES%', extractions)
    # Substitute load variables
    loadvar_names = eqs._diffeq_names + []
    loadvar_names.remove('I') # I is assumed to be a parameter and loaded per time step
    loadvars = '\n    '.join('%SCALAR% ' + name + ' = ' + name + '_arr[neuron_index];' for name in loadvar_names)
    src = src.replace('%LOAD_VARIABLES%', loadvars)
    # Substitute save variables
    savevars = '\n    '.join(name + '_arr[neuron_index] = ' + name + ';' for name in loadvar_names)
    src = src.replace('%STORE_VARIABLES%', savevars)
    # Substitute threshold
    src = src.replace('%THRESHOLD%', threshold)
    # Substitute reset
    reset = '\n            '.join(line.strip() + ';' for line in reset.split('\n') if line.strip())
    src = src.replace('%RESET%', reset)
    # Substitute state update
    sulines = ModelfittingGPUCodeGenerator(dtype=precision).generate(eqs, scheme)
    sulines = re.sub(r'\bdt\b', '%DT%', sulines)
    src = src.replace('%STATE_UPDATE%', sulines.strip())
    # Substitute coincidence counting algorithm
    ccalgo = coincidence_counting_algorithm_src[coincidence_count_algorithm]
    for search, replace in ccalgo.iteritems():
        src = src.replace(search, replace)
    # Substitute dt
    src = src.replace('%DT%', str(float(dt)))
    # Substitute SCALAR
    src = src.replace('%SCALAR%', precision)
    # Substitute number of neurons
    src = src.replace('%NUM_NEURONS%', str(num_neurons))
    # Substitute delta, the coincidence window half-width
    src = src.replace('%DELTA%', str(int(rint(delta / dt))))
    return src


class GPUModelFitting(object):
    '''
    Model fitting class to interface with GPU
    
    Initialisation arguments:
    
    ``G``
        The initialised NeuronGroup to work from.
    ``eqs``
        The equations defining the NeuronGroup.
    ``I``, ``I_offset``
        The current array and offsets (see below).
    ``spiketimes``, ``spiketimes_offset``
        The spike times array and offsets (see below).
    ``spikedelays``,
        Array of delays for each neuron.
    ``refractory``,
        Array of refractory periods, or a single value.
    ``delta``
        The half-width of the coincidence window.
    ``precision``
        Should be 'float' or 'double' - by default the highest precision your
        GPU supports.
    ``coincidence_count_algorithm``
        Should be 'inclusive' if multiple predicted spikes can match one
        target spike, or 'exclusive' (default) if multiple predicted spikes
        can match only one target spike (earliest spikes are matched first). 
    
    Methods:
    
    ``reinit_vars(I, I_offset, spiketimes, spiketimes_offset, spikedelays)``
        Reinitialises all the variables, counters, etc. The state variable values
        are copied from the NeuronGroup G again, and the variables I, I_offset, etc.
        are copied from the method arguments.
    ``launch(duration[, stepsize])``
        Runs the kernel on the GPU for simulation time duration. If ``stepsize``
        is given, the simulation is broken into pieces of that size. This is
        useful on Windows because driver limitations mean that individual GPU kernel
        launches cannot last more than a few seconds without causing a crash.
    
    Attributes:
    
    ``coincidence_count``
        An array of the number of coincidences counted for each neuron.
    ``spike_count``
        An array of the number of spikes counted for each neuron.
    
    **Details**
    
    The equations for the NeuronGroup can be anything, but they will be solved
    with the Euler method. One restriction is that there must be a parameter
    named I which is the time varying input current. (TODO: support for
    multiple input currents? multiple names?)
    
    The current I is passed to the GPU in the following way. Define a single 1D
    array I which contains all the time varying current arrays one after the
    other. Now define an array I_offset of integers so that neuron i should see
    currents: I[I_offset[i]], I[I_offset[i]+1], I[I_offset[i]+2], etc.
    
    The experimentally recorded spike times should be passed in a similar way,
    put all the spike times in a single array and pass an offsets array
    spiketimes_offset. One difference is that it is essential that each spike
    train with the spiketimes array should begin with a spike at a
    large negative time (e.g. -1*second) and end with a spike that is a long time
    after the duration of the run (e.g. duration+1*second). The GPU uses this to
    mark the beginning and end of the train rather than storing the number of
    spikes for each train. 
    '''
    def __init__(self, G, eqs, I, I_offset, spiketimes, spiketimes_offset,
                       spikedelays, refractory,
                       delta, onset=0 * ms,
                       coincidence_count_algorithm='exclusive',
                       precision=default_precision,
                       scheme=euler_scheme
                       ):
        eqs.prepare()
        self.precision = precision
        if precision == 'double':
            self.mydtype = float64
        else:
            self.mydtype = float32
        self.N = N = len(G)
        self.dt = dt = G.clock.dt
        self.delta = delta
        self.onset = onset
        self.eqs = eqs
        self.G = G
        self.coincidence_count_algorithm = coincidence_count_algorithm
        threshold = G._threshold
        if threshold.__class__ is Threshold:
            state = threshold.state
            if isinstance(state, int):
                state = eqs._diffeq_names[state]
            threshold = state + '>' + str(float(threshold.threshold))
        elif isinstance(threshold, VariableThreshold):
            state = threshold.state
            if isinstance(state, int):
                state = eqs._diffeq_names[state]
            threshold = state + '>' + threshold.threshold_state
        elif isinstance(threshold, StringThreshold):
            namespace = threshold._namespace
            expr = threshold._expr
            all_variables = eqs._eq_names + eqs._diffeq_names + eqs._alias.keys() + ['t']
            expr = optimiser.freeze(expr, all_variables, namespace)
            threshold = expr
        else:
            raise ValueError('Threshold must be constant, VariableThreshold or StringThreshold.')
        self.threshold = threshold
        reset = G._resetfun
        if reset.__class__ is Reset:
            state = reset.state
            if isinstance(state, int):
                state = eqs._diffeq_names[state]
            reset = state + ' = ' + str(float(reset.resetvalue))
        elif isinstance(reset, VariableReset):
            state = reset.state
            if isinstance(state, int):
                state = eqs._diffeq_names[state]
            reset = state + ' = ' + reset.resetvaluestate
        elif isinstance(reset, StringReset):
            namespace = reset._namespace
            expr = reset._expr
            all_variables = eqs._eq_names + eqs._diffeq_names + eqs._alias.keys() + ['t']
            expr = optimiser.freeze(expr, all_variables, namespace)
            reset = expr
        self.reset = reset
        self.kernel_src = generate_modelfitting_kernel_src(
                  self.G, eqs, threshold, reset, dt, N, delta,
                  coincidence_count_algorithm=coincidence_count_algorithm,
                  precision=precision, scheme=scheme)

    def reinit_vars(self, I, I_offset, spiketimes, spiketimes_offset,
                    spikedelays, refractory):
        
        self.kernel_module = SourceModule(self.kernel_src)
        self.kernel_func = self.kernel_module.get_function('runsim')

        blocksize = 128
        try:
            blocksize = self.kernel_func.get_attribute(pycuda.driver.function_attribute.MAX_THREADS_PER_BLOCK)
        except: # above won't work unless CUDA>=2.2
            pass
        self.block = (blocksize, 1, 1)
        self.grid = (int(ceil(float(self.N) / blocksize)), 1)
        self.kernel_func_kwds = {'block':self.block, 'grid':self.grid}
        
        mydtype = self.mydtype
        N = self.N
        eqs = self.eqs
        statevars_arr = gpuarray.to_gpu(array(self.G._S.flatten(), dtype=mydtype))
        self.I = gpuarray.to_gpu(array(I, dtype=mydtype))
        self.statevars_arr = statevars_arr
        self.I_offset = gpuarray.to_gpu(array(I_offset, dtype=int32))
        self.spiketimes = gpuarray.to_gpu(array(rint(spiketimes / self.dt), dtype=int32))
        self.spiketime_indices = gpuarray.to_gpu(array(spiketimes_offset, dtype=int32))
        self.num_coincidences = gpuarray.to_gpu(zeros(N, dtype=int32))
        self.spikecount = gpuarray.to_gpu(zeros(N, dtype=int32))
        self.spikedelay_arr = gpuarray.to_gpu(array(rint(spikedelays / self.dt), dtype=int32))
        if isinstance(refractory, float):
            refractory = refractory*ones(N)
        self.refractory_arr = gpuarray.to_gpu(array(rint(refractory / self.dt), dtype=int32))
        self.next_allowed_spiketime_arr = gpuarray.to_gpu(-ones(N, dtype=int32))
        self.next_spike_allowed_arr = gpuarray.to_gpu(ones(N, dtype=bool))
        self.last_spike_allowed_arr = gpuarray.to_gpu(zeros(N, dtype=bool))
        self.kernel_func_args = [self.statevars_arr,
                                 self.I,
                                 self.I_offset,
                                 self.spikecount,
                                 self.num_coincidences,
                                 self.spiketimes,
                                 self.spiketime_indices,
                                 self.spikedelay_arr,
                                 self.refractory_arr,
                                 self.next_allowed_spiketime_arr,
                                 int32(rint(self.onset / self.dt))]
        if self.coincidence_count_algorithm == 'exclusive':
            self.kernel_func_args += [self.last_spike_allowed_arr,
                                      self.next_spike_allowed_arr]

    def launch(self, duration, stepsize=1 * second):
        if stepsize is None:
            self.kernel_func(int32(0), int32(duration / self.dt),
                             *self.kernel_func_args, **self.kernel_func_kwds)
            pycuda.context.synchronize()
        else:
            stepsize = int(stepsize / self.dt)
            duration = int(duration / self.dt)
            for Tstart in xrange(0, duration, stepsize):
                Tend = Tstart + min(stepsize, duration - Tstart)
                self.kernel_func(int32(Tstart), int32(Tend),
                                 *self.kernel_func_args, **self.kernel_func_kwds)
                pycuda.context.synchronize()

    def get_coincidence_count(self):
        return self.num_coincidences.get()

    def get_spike_count(self):
        return self.spikecount.get()
    coincidence_count = property(fget=lambda self:self.get_coincidence_count())
    spike_count = property(fget=lambda self:self.get_spike_count())

if __name__ == '__main__':
    import time
    from matplotlib.cm import jet
    from playdoh import *
    initialise_cuda()
    set_gpu_device(0)
    if 1:
        set_global_preferences(usecodegenthreshold=False)
        N = 10000
        nvar = 100
        delta = 4 * ms
        doplot = True
        eqs = '''
        dV/dt = (-V+R*I)/tau : volt
        Vt : volt
        tau : second
        R : ohm
        I : amp
        '''
        eqs += '\n'.join(['var'+str(i)+':0' for i in range(nvar)])
        eqs = Equations(eqs)
        Vr = 0.0 * volt
        Vt = 1.0 * volt
        I = loadtxt('../../../dev/ideas/cuda/modelfitting/current.txt')
        spiketimes = loadtxt('../../../dev/ideas/cuda/modelfitting/spikes.txt')
        spiketimes -= int(min(spiketimes))
        I_offset = zeros(N, dtype=int)
        spiketimes_offset = zeros(N, dtype=int)
        G = NeuronGroup(N, eqs, reset='V=0*volt; Vt+=1*volt', threshold='V>1*volt')
        G.R = rand(N) * 2e9 + 1e9
        G.tau = rand(N) * 49 * ms + 1 * ms
        spikedelays = rand(N) * 5 * ms
        duration = len(I) * G.clock.dt
        spiketimes = hstack((-1, spiketimes, float(duration) + 1))
        mf = GPUModelFitting(G, eqs, I, I_offset, spiketimes, spiketimes_offset,
                             spikedelays, 0*ms,
                             delta,
                             coincidence_count_algorithm='exclusive')
        mf.reinit_vars(I, I_offset, spiketimes, spiketimes_offset, spikedelays, 0*ms)
        print mf.kernel_src
        start_time = time.time()
        mf.launch(duration)
        running_time = time.time() - start_time

        print 'N:', N
        print 'Duration:', duration
        print 'Total running time:', running_time

        if doplot:

            spikecount = mf.spike_count
            num_coincidences = mf.coincidence_count
            R = G.R
            tau = G.tau

            print 'Spike count varies between', spikecount.min(), 'and', spikecount.max()
            print 'Num coincidences varies between', num_coincidences.min(), 'and', num_coincidences.max()

            subplot(221)
            scatter(R, tau, color=jet(spikecount / float(spikecount.max())))
            xlabel('R')
            ylabel('tau')
            title('Spike count, max = ' + str(spikecount.max()))
            axis('tight')

            subplot(222)
            scatter(R, tau, color=jet(num_coincidences / float(num_coincidences.max())))
            xlabel('R')
            ylabel('tau')
            title('Num coincidences, max = ' + str(num_coincidences.max()))
            axis('tight')

            spikecount -= num_coincidences
            num_coincidences -= spikecount
            num_coincidences[num_coincidences < 0] = 0
            maxcoinc = num_coincidences.max()
            num_coincidences = (1. * num_coincidences) / maxcoinc

            subplot(212)
            scatter(R, tau, color=jet(num_coincidences))
            xlabel('R')
            ylabel('tau')
            title('Hot = ' + str(maxcoinc) + ' excess coincidences, cool = 0 or less')
            axis('tight')

            show()
    if 0:
        N = 10000
        eqs = Equations('''
        dV/dt = (-V+R*I)/tau : volt
        dVt/dt = -(V-1*volt)/tau_t : volt
        R : ohm
        tau : second
        tau_t : second
        Vt_delta : volt
        I : amp
        ''')
        threshold = 'V>Vt'
        reset = '''
        V = 0*volt
        Vt += Vt_delta
        '''
        delta = 4 * ms
        src, declarations_seq = generate_modelfitting_kernel_src(eqs, threshold, reset,
                                               defaultclock.dt, N, delta)
        print src
        print
        print declarations_seq
    if 0:
        # test traces

        #clk = RegularClock(makedefaultclock=True)
        clk = defaultclock

        N = 1
        duration = 200 * ms
        delta = 4 * ms
        Ntarg = 20

        randspikes = hstack(([-1 * second], sort(rand(Ntarg) * duration * .9 + duration * 0.05), [duration + 1 * second]))
        #randspikes = sort(unique(array(randspikes/(2*delta), dtype=int)))*2*delta
        randspikes = sort(unique(array(randspikes / clk.dt, dtype=int))) * clk.dt + 1e-10

        eqs = Equations('''
        dV/dt = (-V+I)/(10*ms) : 1
        I : 1
        ''')
        threshold = 'V>1'
        reset = 'V=0'
        G = NeuronGroup(N, eqs, threshold=threshold, reset=reset, method='Euler',
                        clock=clk)
        #from brian.experimental.ccodegen import *
        #su = AutoCompiledNonlinearStateUpdater(eqs, G.clock, freeze=True)
        #G._state_updater = su
        #I = 1.1*ones(int(duration/defaultclock.dt))
        I = 5.0 * rand(int(duration / defaultclock.dt))
        #I = hstack((zeros(100), 10*ones(int(duration/defaultclock.dt))))
        #I = hstack((zeros(100), 10*ones(100))*(int(duration/defaultclock.dt)/200))
        #I = hstack((zeros(100), 10*exp(-linspace(0,2,100)))*(int(duration/defaultclock.dt)/200))
        #G.I = TimedArray(hstack((0, I)))
        #G.I = TimedArray(I[1:], clock=clk)
        G.I = TimedArray(I, clock=clk)
        M = StateMonitor(G, 'V', record=True, when='end', clock=clk)
        MS = SpikeMonitor(G)
        cc_ex = CoincidenceCounter(source=G, data=randspikes, delta=delta,
                                   coincidence_count_algorithm='exclusive')
        cc_in = CoincidenceCounter(source=G, data=randspikes, delta=delta,
                                   coincidence_count_algorithm='inclusive')
#        cc2 = CoincidenceCounter(source=G, data=randspikes[1:-1], delta=delta)
        run(duration)
        #spiketimes = array([-1*second, duration+1*second])
        #spiketimes_offset = zeros(N, dtype=int)
        #spiketimes = [-1*second]+MS[0]+[duration+1*second]
        spiketimes = randspikes
        spiketimes_offset = zeros(N, dtype=int)
        #I = array([0]+I)
        I_offset = zeros(N, dtype=int)
        spikedelays = zeros(N)
        reinit_default_clock()
        G.V = 0
        mf = GPUModelFitting(G, eqs, I, I_offset, spiketimes, spiketimes_offset,
                             spikedelays,
                             delta,
                             coincidence_count_algorithm='exclusive')
        allV = []
        oldnc = 0
        oldsc = 0
        allcoinc = []
        all_pst = []
        all_nst = []
        allspike = []
        all_nsa = []
        all_lsa = []

        if 0:
            for i in xrange(len(M.times)):
                mf.kernel_func(int32(i), int32(i + 1),
                                 *mf.kernel_func_args, **mf.kernel_func_kwds)
                #autoinit.context.synchronize()
                pycuda.context.synchronize()
                allV.append(mf.state_vars['V'].get())
                all_pst.append(mf.spiketimes.get()[mf.spiketime_indices.get()])
                all_nst.append(mf.spiketimes.get()[mf.spiketime_indices.get() + 1])
                all_nsa.append(mf.next_spike_allowed_arr.get()[0])
                all_lsa.append(mf.last_spike_allowed_arr.get()[0])
#        self.next_spike_allowed_arr = gpuarray.to_gpu(ones(N, dtype=bool))
#        self.last_spike_allowed_arr = gpuarray.to_gpu(zeros(N, dtype=bool))
                nc = mf.coincidence_count[0]
                if nc > oldnc:
                    oldnc = nc
                    allcoinc.append(i * clk.dt)
                sc = mf.spike_count[0]
                if sc > oldsc:
                    oldsc = sc
                    allspike.append(i * clk.dt)

        else:
            mf.launch(duration, stepsize=None)

        print 'Num target spikes:', len(randspikes) - 2
        print 'Predicted spike counts:', MS.nspikes, mf.spike_count[0]

        print 'Coincidences:'
        print 'GPU', mf.coincidence_count
#        print 'CPU bis inc', cc_in.coincidences
        print 'CPU bis exc', cc_ex.coincidences
#        print 'CPU', cc2.coincidences

#        for t in randspikes[1:-1]:
#            plot([t*second-delta, t*second+delta], [0, 0], lw=5, color=(.9,.9,.9))
#        plot(randspikes[1:-1], zeros(len(randspikes)-2), '+', ms=15)
#        plot(M.times, M[0])
        if len(allV):
#            plot(M.times, allV)
#            plot(allcoinc, zeros(len(allcoinc)), 'o')
#            figure()
            plot(M.times, array(all_pst) * clk.dt)
            plot(M.times, array(all_nst) * clk.dt)
            plot(randspikes[1:-1], randspikes[1:-1], 'o')
            plot(allspike, allspike, 'x')
            plot(allcoinc, allcoinc, '+')
            plot(M.times, array(all_nsa) * M.times, '--')
            plot(M.times, array(all_lsa) * M.times, '-.')
            predicted_spikes = allspike
            target_spikes = [t * second for t in randspikes]
            i = 0
            truecoinc = []
            for pred_t in predicted_spikes:
                 while target_spikes[i] < pred_t + delta:
                     if abs(target_spikes[i] - pred_t) < delta:
                         truecoinc.append((pred_t, target_spikes[i]))
                         i += 1
                         break
                     i += 1
            print 'Truecoinc:', len(truecoinc)
            for t1, t2 in truecoinc:
                plot([t1, t2], [t1, t2], ':', color=(0.5, 0, 0), lw=3)
#        show()



# Test of amplifier module
# NOT WORKING
from brian import *
from brian.library.electrophysiology import *

model_clock = Clock(dt=.1 * ms)
sampling_clock = Clock(dt=.1 * ms)

Cm = 200 * pF
Re = 50 * Mohm
Ce = 0.5 * ms / Re
R = 10 * ms / Cm

freq = 100 * Hz
eqs = Equations('''
dv/dt=ie/Cm-v/(10*ms) : volt
I:amp
u=v
''')
eqs += electrode([.6 * Re, .4 * Re], [Ce, Ce], 'vr', 'v', 'ie', 'ic')
eqs += current_clamp('vr', 'ic', 'v_bridge', 'I', bridge=0 * ohm, capa_comp=.95 * Ce)

neuron = NeuronGroup(1, model=eqs, clock=model_clock)
ampli = AcquisitionBoard(neuron, 'v_bridge', 'I', sampling_clock)
mon = StateMonitor(neuron, 'v', record=0, clock=sampling_clock)
mon_vr = StateMonitor(ampli, 'record', record=0, clock=sampling_clock)
mon_I = StateMonitor(ampli, 'command', record=0, clock=sampling_clock)

@network_operation(clock=sampling_clock, when='middle')
def command():
    #neuron.E=sin(sampling_clock.t*2*pi*freq)*10*mV
    ampli.command = rand()*1 * nA - .5 * nA

run(1000 * ms)
v = mon_vr[0] / mV
i = mon_I[0] / nA
K = full_kernel(v, i, 200)
Ke = electrode_kernel_soma(K, 50)
#plot(Ke/Mohm)
print sum(Ke)
#plot(mon.times/ms,mon[0]/mV,'b')
#plot(mon.times/ms,mon_vr[0]/mV,'g')
#show()



'''
An auditory filterbank implemented with Poisson neurons

The input sound has a missing fundamental (only harmonics 2 and 3)
'''
from brian import *

defaultclock.dt = .01 * ms

N = 1500
tau = 1 * ms # Decay time constant of filters = 2*tau
freq = linspace(100 * Hz, 2000 * Hz, N) # characteristic frequencies
f_stimulus = 500 * Hz # stimulus frequency
gain = 500 * Hz

eqs = '''
dv/dt=(-a*w-v+I)/tau : Hz
dw/dt=(v-w)/tau : Hz # e.g. linearized potassium channel with conductance a
a : 1
I = gain*(sin(4*pi*f_stimulus*t)+sin(6*pi*f_stimulus*t)) : Hz
'''

neurones = NeuronGroup(N, model=eqs, threshold=PoissonThreshold())
neurones.a = (2 * pi * freq * tau) ** 2

spikes = SpikeMonitor(neurones)
counter = SpikeCounter(neurones)
run(100 * ms)

subplot(121)
CF = array([freq[i] for i, _ in spikes.spikes])
timings = array([t for _, t in spikes.spikes])
plot(timings / ms, CF, '.')
xlabel('Time (ms)')
ylabel('Characteristic frequency (Hz)')
subplot(122)
plot(counter.count / (300 * ms), freq)
xlabel('Firing rate (Hz)')
show()



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

#x = Sound(randn(15,2), samplerate=100*Hz)
#
#print x
#print x.buffer_fetch(0, 10)
#print x.buffer_fetch(5, 15)
#print x.buffer_fetch(10, 20)

#y = Sound(0.1*randn(200,2), samplerate=100*Hz)
#
#x = x+y
#x = x.shift(500*ms)
#x = Sound.sequence((x, x))
#print x.intensity('peak')
#x = x.repeat(2)
#x.copy_from(y)

x = Sound.tone(500*Hz, 500*ms)
y = Sound.tone(5000*Hz, 500*ms)
x, y = y, x
#z = Sound(array([x.flatten(), y.flatten()]).T, samplerate=x.samplerate)
z = Sound((x, y), samplerate=x.samplerate)
z.play(True, True)
x=z

#x = Sound([lambda t:sin(2*pi*t*50),
#           lambda t:cos(2*pi*t*100)], samplerate=10000*Hz, duration=1*second)

#x = Sound(randn(100,2), samplerate=100*Hz).ramped(duration=200*ms)
#x = Sound.tone(500*Hz, 500*ms)

#for i in xrange(x.nchannels):
#    plot(x.times, x.channel(i))
#show()

#x = Sound(randn(44100,2), samplerate=44100*Hz)
#x.left.spectrogram()
#show()


'''
Short-term synaptic plasticity.

Implements the short-term plasticity model described in:
Markram et al (1998). Differential signaling via the same axon of
neocortical pyramidal neurons, PNAS. Synaptic dynamics is
described by two variables x and u, which follows the following differential equations::

  dx/dt=(1-x)/taud  (depression)
  du/dt=(U-u)/tauf  (facilitation)

where taud, tauf are time constants and U is a parameter in 0..1. Each presynaptic
spike triggers modifications of the variables::

  x<-x*(1-u)
  u<-u+U*(1-u)

Synaptic weights are modulated by the product u*x (in 0..1) (before update).
'''
# See BEP-1

from network import NetworkOperation
from neurongroup import NeuronGroup
from monitor import SpikeMonitor
from scipy import zeros, exp, isscalar
from connections import DelayConnection

__all__ = ['STP']


class STPGroup(NeuronGroup):
    '''
    Neuron group forwarding spikes with short term plasticity modulation.
    '''
    def __init__(self, N, clock=None):
        eqs = '''
        ux : 1
        x : 1
        u : 1
        '''
        NeuronGroup.__init__(self, N, model=eqs, clock=clock)

    def update(self):
        pass


class STPUpdater(SpikeMonitor):
    '''
    Event-driven updates of STP variables.
    '''
    def __init__(self, source, P, taud, tauf, U, delay=0):
        SpikeMonitor.__init__(self, source, record=False, delay=delay)
        # P is the group with the STP variables
        N = len(P)
        self.P = P
        self.minvtaud = -1. / taud
        self.minvtauf = -1. / tauf
        self.U = U
        self.ux = P.ux
        self.x = P.x
        self.u = P.u
        self.lastt = zeros(N) # last update
        self.clock = P.clock

    def propagate(self, spikes):
        interval = self.clock.t - self.lastt[spikes]
        self.u[spikes] = self.U + (self.u[spikes] - self.U) * exp(interval * self.minvtauf)
        tmp = 1 - self.u[spikes]
        self.x[spikes] = 1 + (self.x[spikes] - 1) * exp(interval * self.minvtaud)
        self.ux[spikes] = self.u[spikes] * self.x[spikes]
        self.x[spikes] *= tmp
        self.u[spikes] += self.U * tmp
        self.lastt[spikes] = self.clock.t
        self.P.LS.push(spikes)


class STPUpdater2(STPUpdater):
    '''
    STP Updater where U, taud and tauf are vectors
    '''
    def propagate(self, spikes):
        interval = self.clock.t - self.lastt[spikes]
        self.u[spikes] = self.U[spikes] + (self.u[spikes] - self.U[spikes]) * exp(interval * self.minvtauf[spikes])
        tmp = 1 - self.u[spikes]
        self.x[spikes] = 1 + (self.x[spikes] - 1) * exp(interval * self.minvtaud[spikes])
        self.ux[spikes] = self.u[spikes] * self.x[spikes]
        self.x[spikes] *= tmp
        self.u[spikes] += self.U[spikes] * tmp
        self.lastt[spikes] = self.clock.t
        self.P.LS.push(spikes)


class SynapticDepressionUpdater(SpikeMonitor):
    '''
    Event-driven updates of STP variables.
    Special case: tauf=0*ms (synaptic depression).
    
      dx/dt=(1-x)/taud  (depression)
      x<-x*(1-U)

    NOT FINISHED
    '''
    def __init__(self, source, P, taud, tauf, U, delay=0):
        SpikeMonitor.__init__(self, source, record=False, delay=delay)
        # P is the group with the STP variables
        N = len(P)
        self.P = P
        self.minvtaud = -1. / taud
        self.U = U
        self.ux = P.ux
        self.x = P.x
        self.lastt = zeros(N) # last update
        self.clock = P.clock

    def propagate(self, spikes):
        interval = self.clock.t - self.lastt[spikes]
        self.x[spikes] = 1 + (self.x[spikes] - 1) * exp(interval * self.minvtaud)
        self.ux[spikes] = self.U * self.x[spikes]
        self.x[spikes] *= 1 - self.U
        self.lastt[spikes] = self.clock.t
        self.P.LS.push(spikes)


class STP(NetworkOperation):
    '''
    Short-term synaptic plasticity, following the Tsodyks-Markram model.

    Implements the short-term plasticity model described in Markram et al (1998).
    Differential signaling via the same axon of
    neocortical pyramidal neurons, PNAS.
    Synaptic dynamics is described by two variables x and u, which follow
    the following differential equations::
    
      dx/dt=(1-x)/taud  (depression)
      du/dt=(U-u)/tauf  (facilitation)
    
    where taud, tauf are time constants and U is a parameter in 0..1. Each presynaptic
    spike triggers modifications of the variables::
    
      u<-u+U*(1-u)
      x<-x*(1-u)
    
    Synaptic weights are modulated by the product ``u*x`` (in 0..1) (before update).
    
    Reference:
    
    * Markram et al (1998). "Differential signaling via the same axon of
      neocortical pyramidal neurons", PNAS.
    '''
    def __init__(self, C, taud, tauf, U):
        if isinstance(C, DelayConnection):
            raise AttributeError, "STP does not handle heterogeneous connections yet."
        NetworkOperation.__init__(self, lambda:None, clock=C.source.clock)
        N = len(C.source)
        P = STPGroup(N, clock=C.source.clock)
        P.x = 1
        P.u = U
        P.ux = U
        if (isscalar(taud) & isscalar(tauf) & isscalar(U)):
            updater = STPUpdater(C.source, P, taud, tauf, U, delay=C.delay * C.source.clock.dt)
        else:
            updater = STPUpdater2(C.source, P, taud, tauf, U, delay=C.delay * C.source.clock.dt)
        self.contained_objects = [updater]
        C.source = P
        C.delay = 0
        C._nstate_mod = 0 # modulation of synaptic weights
        self.vars = P

    def __call__(self):
        pass



from brian import *
from nose.tools import *
from brian.utils.approximatecomparisons import is_approx_equal, is_within_absolute_tolerance
try:
    from brian.experimental.cuda.gpu_modelfitting import *
    import pycuda.autoinit as autoinit
    use_gpu = True
except ImportError:
    use_gpu = False

def test_spikemonitor():
    '''
    :class:`SpikeMonitor`
    ~~~~~~~~~~~~~~~~~~~~~
    
    Records spikes from a :class:`NeuronGroup`. Initialised as one of::
    
        SpikeMonitor(source(,record=True))
        SpikeMonitor(source,function=function)
    
    Where:
    
    source
        A :class:`NeuronGroup` to record from
    record
        True or False to record all the spikes or just summary
        statistics.
    function
        A function f(spikes) which is passed the array of spikes
        numbers that have fired called each step, to define
        custom spike monitoring.
    
    Has two attributes:
    
    nspikes
        The number of recorded spikes
    spikes
        A time ordered list of pairs (i,t) where neuron i fired
        at time t.
    
    :class:`StateMonitor`
    ~~~~~~~~~~~~~~~~~~~~~
    
    Records the values of a state variable from a :class:`NeuronGroup`.
    Initialise as::
    
        StateMonitor(P,varname(,record=False)
            (,when='end)(,timestep=1)(,clock=clock))
    
    Where:
    
    P
        The group to be recorded from
    varname
        The state variable name or number to be recorded
    record
        What to record. The default value is False and the monitor will
        only record summary statistics for the variable. You can choose
        record=integer to record every value of the neuron with that
        number, record=list of integers to record every value of each of
        those neurons, or record=True to record every value of every
        neuron (although beware that this may use a lot of memory).
    when
        When the recording should be made in the :class:`Network` update, possible
        values are any of the strings: 'start', 'before_groups', 'after_groups',
        'before_connections', 'after_connections', 'before_resets',
        'after_resets', 'end' (in order of when they are run).
    timestep
        A recording will be made each timestep clock updates (so timestep
        should be an integer).
    clock
        A clock for the update schedule, use this if you have specified a
        clock other than the default one in your network, or to update at a
        lower frequency than the update cycle. Note though that if the clock
        here is different from the main clock, the when parameter will not
        be taken into account, as network updates are done clock by clock.
        Use the timestep parameter if you need recordings to be made at a
        precise point in the network update step.

    The :class:`StateMonitor` object has the following properties:

    times
        The times at which recordings were made
    mean
        The mean value of the state variable for every neuron in the
        group (not just the ones specified in the record keyword)
    var
        The unbiased estimate of the variances, as in mean
    std
        The square root of var, as in mean
        
    In addition, if M is a :class:`StateMonitor` object, you write::
    
        M[i]
    
    for the recorded values of neuron i (if it was specified with the
    record keyword). It returns an array object.
    
    Others
    ~~~~~~
    
    The following monitors also exist, but are not part of the
    assured interface because their syntax is subject to change. See the documentation
    for each class for more details.
    
    * :class:`Monitor` (base class)
    * :class:`ISIHistogramMonitor`
    * :class:`FileSpikeMonitor`
    * :class:`PopulationRateMonitor`
    '''
    reinit_default_clock()

    # test that SpikeMonitor retrieves the spikes generator by SpikeGeneratorGroup

    spikes = [(0, 3 * ms), (1, 4 * ms), (0, 7 * ms)]

    G = SpikeGeneratorGroup(2, spikes, clock=defaultclock)
    M = SpikeMonitor(G)
    net = Network(G, M)
    net.run(10 * ms)

    assert (M.nspikes == 3)
    for (mi, mt), (i, t) in zip(M.spikes, spikes):
        assert (mi == i)
        assert (is_approx_equal(mt, t))

    # test that the spiketimes are saved and accessed correctly
    
    assert(len(M[0]) == len(M.spiketimes[0]) == 2 and 
           len(M[1]) == len(M.spiketimes[1]) == 1)
    assert((M.spiketimes[0] == M[0]).all() and (M.spiketimes[1] == M[1]).all())

    # test that spiketimes are cleared on reinit
    
    M.reinit()
    assert (M.nspikes == 0)
    assert (len(M.spikes) == 0)
    assert (len(M[0]) == 0 and len(M[1]) == 0)
    assert (len(M.spiketimes[0]) == 0 and len(M.spiketimes[1]) == 0)

    # test that SpikeMonitor function calling usage does what you'd expect    

    f_spikes = []

    def f(spikes):
        if len(spikes):
            f_spikes.extend(spikes)

    G = SpikeGeneratorGroup(2, spikes, clock=defaultclock)
    M = SpikeMonitor(G, function=f)
    net = Network(G, M)
    reinit_default_clock()
    net.run(10 * ms)
    assert (f_spikes == [0, 1, 0])

    # test that SpikeMonitors in MultiConnection objects do reinitialize
    # properly
    G = SpikeGeneratorGroup(2, spikes, clock=defaultclock)
    G2 = NeuronGroup(1, model='dv/dt = -v / (5*ms) : 1')
    C = Connection(G, G2, 'v', weight=0)
    M = SpikeMonitor(G)
    # Note: Because M and C share the source (G), they are replaced by a 
    #       MultiConnection 
    net = Network(G, G2, C, M)
    net.run(10 * ms)
    
    net.reinit() # make sure that the reinit propagates to the SpikeMonitor
    assert (M.nspikes == 0)
    assert (len(M.spikes) == 0)
    assert (len(M[0]) == 0 and len(M[1]) == 0)
    assert (len(M.spiketimes[0]) == 0 and len(M.spiketimes[1]) == 0)
    
    
    

    # test interface for StateMonitor object

    dV = 'dV/dt = 0*Hz : 1.'
    G = NeuronGroup(3, model=dV, reset=0., threshold=10.)
    @network_operation(when='start')
    def f(clock):
        if clock.t >= 1 * ms:
            G.V = [1., 2., 3.]
    M1 = StateMonitor(G, 'V')
    M2 = StateMonitor(G, 'V', record=0)
    M3 = StateMonitor(G, 'V', record=[0, 1])
    M4 = StateMonitor(G, 'V', record=True)
    reinit_default_clock()
    net = Network(G, f, M1, M2, M3, M4)
    net.run(2 * ms)
    assert (is_within_absolute_tolerance(M2[0][0], 0.))
    assert (is_within_absolute_tolerance(M2[0][-1], 1.))
    assert (is_within_absolute_tolerance(M3[1][0], 0.))
    assert (is_within_absolute_tolerance(M3[1][-1], 2.))
    assert (is_within_absolute_tolerance(M4[2][0], 0.))
    assert (is_within_absolute_tolerance(M4[2][-1], 3.))
    assert_raises(IndexError, M1.__getitem__, 0)
    assert_raises(IndexError, M2.__getitem__, 1)
    assert_raises(IndexError, M3.__getitem__, 2)
    assert_raises(IndexError, M4.__getitem__, 3)
    for M in [M3, M4]:
        assert (is_within_absolute_tolerance(float(max(abs(M.times - M2.times))), float(0 * ms)))
        assert (is_within_absolute_tolerance(float(max(abs(M.times_ - M2.times_))), 0.))
    assert (is_within_absolute_tolerance(float(M2.times[0]), float(0 * ms)))
    d = diff(M2.times)
    assert (is_within_absolute_tolerance(max(d), min(d)))
    assert (is_within_absolute_tolerance(float(max(d)), float(get_default_clock().dt)))
    # construct unbiased estimator from variances of recorded arrays
    v = array([ var(M4[0]), var(M4[1]), var(M4[2]) ]) * float(len(M4[0])) / float(len(M4[0]) - 1)
    m = array([0.5, 1.0, 1.5])
    assert (is_within_absolute_tolerance(abs(max(M1.mean - m)), 0.))
    assert (is_within_absolute_tolerance(abs(max(M1.var - v)), 0.))
    assert (is_within_absolute_tolerance(abs(max(M1.std - v ** 0.5)), 0.))

    # test when, timestep, clock for StateMonitor
    c = Clock(dt=0.1 * ms)
    cslow = Clock(dt=0.2 * ms)
    dV = 'dV/dt = 0*Hz : 1.'
    G = NeuronGroup(1, model=dV, reset=0., threshold=1., clock=c)
    @network_operation(when='start', clock=c)
    def f():
        G.V = 2.
    M1 = StateMonitor(G, 'V', record=True, clock=cslow)
    M2 = StateMonitor(G, 'V', record=True, timestep=2, clock=c)
    M3 = StateMonitor(G, 'V', record=True, when='before_groups', clock=c)
    net = Network(G, f, M1, M2, M3, M4)
    net.run(2 * ms)
    print M1[0], M3[0]
    assert (2 * len(M1[0]) == len(M3[0]))
    assert (len(M1[0]) == len(M2[0]))
    for i in range(len(M1[0])):
        assert (is_within_absolute_tolerance(M1[0][i], M2[0][i]))
        assert (is_within_absolute_tolerance(M1[0][i], 0.))
    for x in M3[0]:
        assert (is_within_absolute_tolerance(x, 2.))

    reinit_default_clock() # for next test

def test_counter():
    '''
    Tests the consistency of :class:`SpikeCounter`,
    :class:`PopulationSpikeCounter` and :class:`SpikeMonitor`.
    '''
    
    reinit_default_clock()
    
    # Test whether all monitors count the same number of spikes
    
    spikes = [(0, 3 * ms), (1, 4 * ms), (0, 7 * ms)]

    G = SpikeGeneratorGroup(2, spikes, clock=defaultclock)
    M = SpikeMonitor(G)
    C = SpikeCounter(G)
    P = PopulationSpikeCounter(G)
    net = Network(G, M, C, P)
    net.run(10 * ms)
    
    # total number of spikes
    assert(M.nspikes == C.nspikes == P.nspikes == 3)
    # spikes per neuron
    assert(len(M[0]) == C[0] == C.count[0] == 2)
    assert(len(M[1]) == C[1] == C.count[1] == 1)
    # check for correct reinit
    net.reinit()
    assert(M.nspikes == C.nspikes == P.nspikes == 0)
    assert(len(M[0]) == C[0] == C.count[0] == 0)
    assert(len(M[1]) == C[1] == C.count[1] == 0)
    
    reinit_default_clock() # for next test
    

def test_coincidencecounter():
    """
    Simulates an IF model with constant input current and checks
    the total number of coincidences with prediction.
    """

    eqs = """
    dV/dt = (-V+R*I)/tau : 1
    I : 1
    R : 1
    tau : second
    """
    reset = 0
    threshold = 1

    duration = 500 * ms
    input = 1.2 + .2 * randn(int(duration / defaultclock._dt))
    delta = 4 * ms
    n = 10

    def get_data(n):
        # Generates data from an IF neuron
        group = NeuronGroup(N=1, model=eqs, reset=reset, threshold=threshold,
                            method='Euler', refractory=3 * delta)
        group.I = TimedArray(input, start=0 * second, dt=defaultclock.dt)
        group.R = 1.0
        group.tau = 20 * ms
        M = SpikeMonitor(group)
        stM = StateMonitor(group, 'V', record=True)
        net = Network(group, M, stM)
        net.run(duration)
        data = M.spikes
#        train0 = M.spiketimes[0]
        reinit_default_clock()

#        trains = []
#        for i in range(n):
#            trains += zip(i*ones(len(train0), dtype='int'), (array(train0) + (delta*c) * rand(len(train0))))
#        trains.sort(lambda x,y: (2*int(x[1]>y[1])-1))
#        trains = [(i,t*second) for i,t in trains]

        return data, stM.values#, trains

    data, data_voltage = get_data(n=n)
    train0 = [t for i, t in data]

    group = NeuronGroup(n, eqs, reset=reset, threshold=threshold,
                        method='Euler')
    group.I = TimedArray(input, start=0 * second, dt=defaultclock.dt)
    group.R = 1.0 * ones(n)
    group.tau = 20 * ms * (1 + .1 * (2 * rand(n) - 1))

    cc = CoincidenceCounter(source=group, data=([-1 * second] + train0 + [duration + 1 * second]), delta=delta)
    sm = SpikeMonitor(group)
    statem = StateMonitor(group, 'V', record=True)
    net = Network(group, cc, sm, statem)
    net.run(duration)
    reinit_default_clock()

    cpu_voltage = statem.values

    online_coincidences = cc.coincidences
    cpu_spike_count = array([len(sm[i]) for i in range(n)])
    offline_coincidences = array([gamma_factor(sm[i], train0, delta=delta, normalize=False, dt=defaultclock.dt) for i in range(n)])

    if use_gpu:
        # Compute gamma factor with GPU
        inp = array(input)
        I_offset = zeros(n, dtype=int)
        #spiketimes = array(hstack(([-1*second],train0,[data[-1][1]+1*second])))
        spiketimes = array(hstack(([-1 * second], train0, [duration + 1 * second])))
        spiketimes_offset = zeros(n, dtype=int)
        spikedelays = zeros(n)
        cd = CoincidenceCounter(source=group, data=data, delta=delta)
        group.V = 0.0

        mf = GPUModelFitting(group, Equations(eqs),
                             inp, I_offset, spiketimes, spiketimes_offset,
                             spikedelays, delta)

        # Normal GPU launch
        mf.launch(duration)

        # GPU record of voltage and spikes 
    #    allV = []
    #    oldnc = 0
    #    oldsc = 0
    #    allcoinc = []
    #    all_pst = []
    #    all_nst = []
    #    allspike = []
    #    all_nsa = []
    #    all_lsa = []
    #    
    #    for i in xrange(int(duration/defaultclock.dt)):
    #        mf.kernel_func(int32(i), int32(i+1),
    #                         *mf.kernel_func_args, **mf.kernel_func_kwds)
    #        autoinit.context.synchronize()
    #        allV.append(mf.state_vars['V'].get())
    #        all_pst.append(mf.spiketimes.get()[mf.spiketime_indices.get()])
    #        all_nst.append(mf.spiketimes.get()[mf.spiketime_indices.get()+1])
    #        all_nsa.append(mf.next_spike_allowed_arr.get()[0])
    #        all_lsa.append(mf.last_spike_allowed_arr.get()[0])
    #    #        self.next_spike_allowed_arr = gpuarray.to_gpu(ones(N, dtype=bool))
    #    #        self.last_spike_allowed_arr = gpuarray.to_gpu(zeros(N, dtype=bool))
    #        nc = mf.coincidence_count[0]
    #        if nc>oldnc:
    #            oldnc = nc
    #            allcoinc.append(i*defaultclock.dt)
    #        sc = mf.spike_count[0]
    #        if sc>oldsc:
    #            oldsc = sc
    #            allspike.append(i*defaultclock.dt)
    #    
    #    gpu_voltage = array(allV)


        cc = mf.coincidence_count
        gpu_spike_count = mf.spike_count
        cd._model_length = gpu_spike_count
        cd._coincidences = cc
        gpu_coincidences = cc

    print "Spike count"
    print "Data", len(data)
    print "CPU", cpu_spike_count
    if use_gpu:
        print "GPU", gpu_spike_count
        print "max error : %.1f" % max(abs(cpu_spike_count - gpu_spike_count))
        print
    print "Offline"
    print offline_coincidences
    print
    print "Online"
    print online_coincidences
    print "max error : %.6f" % max(abs(online_coincidences - offline_coincidences))
    if use_gpu:
        print
        print "GPU"
        print gpu_coincidences
        print "max error : %.6f" % max(abs(gpu_coincidences - offline_coincidences))

        bad_neuron = nonzero(abs(gpu_coincidences - offline_coincidences) > 1e-10)[0]
        if len(bad_neuron) > 0:
            print "Bad neuron", bad_neuron, group.tau[bad_neuron[0]]
    print
    print
    return

#    plot(linspace(0,duration/second,len(data_voltage[0])), data_voltage[0], 'k', linewidth=.5)
#    plot(linspace(0,duration/second,len(cpu_voltage[0])), cpu_voltage[0], 'b')
#    plot(linspace(0,duration/second,len(gpu_voltage[:,0])), gpu_voltage[:,0], 'g')
#    show()
    times = linspace(0, duration / second, len(all_pst))

    figure()

    plot(times, array(all_pst) * defaultclock.dt)
    plot(times, array(all_nst) * defaultclock.dt)
    plot(train0, train0, 'o')
    plot(allspike, allspike, 'x')
    plot(allcoinc, allcoinc, '+')
    plot(times, array(all_nsa) * times, '--')
    plot(times, array(all_lsa) * times, '-.')
    predicted_spikes = allspike
    target_spikes = train0 + [duration + 1 * second]
    i = 0
    truecoinc = []
    for pred_t in predicted_spikes:
         while target_spikes[i] <= pred_t + delta - 1e-10 * second:
             if abs(target_spikes[i] - pred_t) <= delta + 1e-10 * second:
                 truecoinc.append((pred_t, target_spikes[i]))
                 i += 1
                 break
             i += 1
    print 'Truecoinc:', len(truecoinc)
    for t1, t2 in truecoinc:
        plot([t1, t2], [t1, t2], ':', color=(0.5, 0, 0), lw=3)

    show()

#    assert is_within_absolute_tolerance(online_gamma1,offline_gamma1)    
#    assert is_within_absolute_tolerance(online_gamma2,offline_gamma2)   


#def test_vectorized_spikemonitor():
#    eqs = """
#    dV/dt = (-V+I)/tau : 1
#    tau : second
#    I : 1
#    """ 
#    N = 30
#    taus = 10*ms + 90*ms * rand(N)
#    duration = 1000*ms
#    input = 2.0 + 3.0 * rand(int(duration/defaultclock._dt))
#    vgroup = VectorizedNeuronGroup(model=eqs, reset=0, threshold=1,
#                                   input=input, slices=2, overlap=200*ms, tau=taus)
#    M = SpikeMonitor(vgroup)
#    run(vgroup.duration)
#    raster_plot(M)
#    show()

if __name__ == '__main__':
    test_spikemonitor()
    test_counter()
#    test_coincidencecounter()



'''
Attempting to use chag:pp CUDA library with pycuda or with weave.
This doesn't work.

Message on PyCUDA list follows:

On Dienstag 29 September 2009, Dan Goodman wrote:
> > Hi all,
> > 
> > Has anyone tried using pycuda with a library of GPU code such as CUDPP
> > [1] or chag:pp [2]? How do you use these in pycuda? Is it just a matter
> > of passing some header and lib file directories to nvcc via the
> > options=... keyword for SourceModule?

SourceModule won't work because it's meant only for device-side code, and most 
of these libraries expose host-side interfaces.

If the library is written with the driver API in mind, it's as simple as 
wrapping it into Python using, e.g. Boost.Python, cython, etc. If it is using 
the CUDA "runtime" API, it's not quite as simple, because then Nvidia's 
idiotic driver/runtime dichotomy kicks in and prevents direct communication. 
Luckily, however, I learned at the Nvidia conference last week that a) they're 
aware that this is causing problems, and b) this issue will go away in CUDA 
3.0, which will be released along with their new hardware.

HTH,
Andreas

'''

import pycuda.autoinit as autoinit
import pycuda.driver as drv
from pycuda.gpuarray import GPUArray
from pycuda import gpuarray
import bisect
import numpy, pylab, time, random
from scipy import weave

N = 1024

mod = drv.SourceModule('''
#include <chag/pp/compact.cuh>

__global__ void test()
{
 int i = blockIdx.x * blockDim.x + threadIdx.x;
 i = i+1;
}
''', options=[r'-I"C:\Documents and Settings\goodman.CELERI\Bureau\source-20090929"'],
     no_extern_c=True)

#x = numpy.random.randn(N)
#xgpu = gpuarray.to_gpu(x)
#ygpu = gpuarray.to_gpu(numpy.zeros(x.shape))
#vec = gpuarray.to_gpu(numpy.zeros(1, dtype=int))
#z = x[x>=0.0]
#
#xgpu_start = int(xgpu.gpudata)
#xgpu_end = xgpu_start+N*8
#ygpu_start = int(ygpu.gpudata)
#vec_start = int(vec.gpudata)
#
#src = '''
#struct Predicate
#{
#    __device__ bool operator() (double value) const
#    {
#        return value>0.0;
#    }
#}
#
#pp::compact(
#    (double *)xgpu_start,              /* Input start pointer */
#    (double *)xgpu_end,     /* Input end pointer */
#    (double *)ygpu_start,              /* Output start pointer */
#    vec_start,            /* Storage for valid element count */
#    Predicate()             /* Predicate */
#    );      
#'''
#
#weave.inline(src, ['xgpu_start', 'xgpu_end', 'ygpu_start', 'vec_start'],
#             compiler='gcc',#msvc works too
#             headers=[
#                      '<chag/pp/compact.cuh>'],
#             include_dirs=['C:\\CUDA\\include',
#                           r'C:\Documents and Settings\goodman.CELERI\Bureau\source-20090929'],
#             libraries=[],
#             library_dirs=['C:\\CUDA\\lib'],
#             )



"""
Reliability of spike timing
---------------------------
Adapted from Fig. 10D,E of
Brette R and E Guigon (2003). Reliability of Spike Timing Is a General Property
of Spiking Model Neurons. Neural Computation 15, 279-308.

This shows that reliability of spike timing is a generic property of spiking
neurons, even those that are not leaky.
This is a non-physiological model which can be leaky or anti-leaky depending
on the sign of the input I.

All neurons receive the same fluctuating input, scaled by a parameter p that
varies across neurons. This shows:

1. reproducibility of spike timing
2. robustness with respect to deterministic changes (parameter)
3. increased reproducibility in the fluctuation-driven regime (input crosses
   the threshold)
"""
from brian import *

N=500
tau=33*ms
taux=20*ms
sigma=0.02

eqs_input='''
B=2./(1+exp(-2*x))-1 : 1
dx/dt=-x/taux+(2/taux)**.5*xi : 1
'''

eqs='''
dv/dt=(v*I+1)/tau + sigma*(2/tau)**.5*xi : 1
I=0.5+3*p*B : 1
B : 1
p : 1
'''

input=NeuronGroup(1,eqs_input)
neurons=NeuronGroup(N,eqs,threshold=1,reset=0)
neurons.p=linspace(0,1,N)
neurons.v=rand(N)
neurons.B=linked_var(input,'B')

M=StateMonitor(input,'B',record=0)
S=SpikeMonitor(neurons)

run(1000*ms)

subplot(211) # The input
plot(M.times/ms,M[0])
subplot(212)
raster_plot(S)
plot([0,1000],[250,250],'r')
show()



'''
Brette, R. (2008). Generation of correlated spike trains.

Response of a leaky integrate-and-fire model to correlated inputs.
Comparison with Ruben's equations.

Fig 8 in the paper.
'''

from brian import *
from time import time

# Firing rate of an integrate-and-fire model with white noise input current
fWNIF = lambda x: exp(x ** 2) * (1. + special.erf(x))
def rateIFWhiteNoise(refrac, tau, theta_eff, H_eff):
    a, _ = integrate.quad(fWNIF, H_eff, theta_eff)
    return 1. / (refrac + a * tau * sqrt(pi))

# Two independently correlated populations (no correlation between the two)
# ** Doubly stochastic processes **
Ne = 800
Ni = 200
tauc = 1 * ms
Je = 0.04
Ji = -0.16
nu = 10. * Hz
duration = 20 * second
c = 0.001 # 0.02
taum = 10. * ms
refrac = 2. * ms

# Scaling
scaling = 20.
nu = nu * scaling
Je = Je / sqrt(scaling)
Ji = Ji / sqrt(scaling)

'''
**********
Simulation
**********
'''
name = 'mixtureB'
if name == 'Cox':
    # Cox processes
    input_exc = HomogeneousCorrelatedSpikeTrains(Ne, nu, c, tauc)
    input_inh = HomogeneousCorrelatedSpikeTrains(Ni, nu, c, tauc)
elif name == 'mixtureA':
    # Mixture processes
    Te = mixture_process(nu=array([nu / c]), P=ones((Ne, 1)) * c, tauc=tauc, t=duration)
    Ti = mixture_process(nu=array([nu / c]), P=ones((Ni, 1)) * c, tauc=tauc, t=duration)
    input_exc = SpikeGeneratorGroup(Ne, Te)
    input_inh = SpikeGeneratorGroup(Ni, Ti)
elif name == 'mixtureB':
    # Mixture processes
    Pe = eye(Ne, Ne + 1)
    Pe[:, Ne] = 1
    Pi = eye(Ni, Ni + 1)
    Pi[:, Ni] = 1
    Te = mixture_process(nu=array([c * nu] * Ne + [(1 - c) * nu]), P=Pe, tauc=tauc, t=duration)
    Ti = mixture_process(nu=array([c * nu] * Ni + [(1 - c) * nu]), P=Pi, tauc=tauc, t=duration)
    input_exc = SpikeGeneratorGroup(Ne, Te)
    input_inh = SpikeGeneratorGroup(Ni, Ti)

neuron = NeuronGroup(1, model='dv/dt=-v/taum : 1', reset=0, threshold=1, refractory=refrac)
exc_synapses = Connection(input_exc, neuron, 'v')
exc_synapses.connect_full(input_exc, neuron, Je)
inh_synapses = Connection(input_inh, neuron, 'v')
inh_synapses.connect_full(input_inh, neuron, Ji)
counter = SpikeCounter(neuron)

t1 = time()
run(duration)
t2 = time()
print "Simulated in", t2 - t1, "s"
print "Rate:", counter[0] / duration

'''
***********************
Theoretical predictions
***********************
'''
# Ruben's formulas
F = c + 1. # Fano factor
rho = c / F # Spike count correlation
sigmaw2 = Je * Je * Ne * nu + Ji * Ji * Ni * nu
sigma2 = Je * Je * nu * Ne * (F - 1. + (Ne - 1.) * F * rho) + Ji * Ji * nu * Ni * (F - 1. + (Ni - 1.) * F * rho)

alpha = sigma2 / sigmaw2 # correlation magnitude
mu = Je * Ne * nu + Ji * Ni * nu
sigmaeff = sqrt(sigmaw2 + sigma2)
sigmaw = sqrt(sigmaw2)
thetahat = (1. - mu * taum) / (sigmaw * sqrt(taum))
Hhat = -mu * taum / (sigmaw * sqrt(taum))

print "Fano factor =", F
print "Spike count correlation (rho) =", rho
print "Correlation magnitude (alpha) =", alpha
print "Mean input =", mu
print "Input variance =", sigma2

nueff = rateIFWhiteNoise(refrac, taum, (1. - mu * taum) / (sigmaeff * sqrt(taum)), -mu * taum / (sigmaeff * sqrt(taum)))
nu0 = rateIFWhiteNoise(refrac, taum, thetahat, Hhat)

print "Theoretical rate for uncorrelated inputs =", nu0
print "Theoretical rate for instantaneous correlations =", nueff

# coefficient for Ruben's long tauc formula
C = alpha * (taum * nu0) ** 2 * (taum * nu0 * (fWNIF(thetahat) - fWNIF(Hhat)) ** 2 / (1. - nu0 * refrac) - \
            (thetahat * fWNIF(thetahat) - Hhat * fWNIF(Hhat)) / sqrt(2))

print "C =", C

shortprediction = nueff - alpha * nu0 * nu0 * sqrt(.5 * pi * taum * tauc) * fWNIF((1. - mu * taum) / (sigmaw * sqrt(taum)))
longprediction = nu0 + C / tauc

print "Predicted rate for short tauc:", shortprediction
print "Predicted rate for long tauc:", longprediction



'''
Spike-based adaptation of Licklider's model of pitch processing (autocorrelation with
delay lines) with phase locking.

Romain Brette
'''
from brian import *

defaultclock.dt = .02 * ms

# Ear and sound
max_delay = 20 * ms # 50 Hz
tau_ear = 1 * ms
sigma_ear = .1
eqs_ear = '''
dx/dt=(sound-x)/tau_ear+sigma_ear*(2./tau_ear)**.5*xi : 1
sound=5*sin(2*pi*frequency*t)**3 : 1 # nonlinear distorsion
#sound=5*(sin(4*pi*frequency*t)+.5*sin(6*pi*frequency*t)) : 1 # missing fundamental
frequency=(200+200*t*Hz)*Hz : Hz # increasing pitch
'''
receptors = NeuronGroup(2, model=eqs_ear, threshold=1, reset=0, refractory=2 * ms)
traces = StateMonitor(receptors, 'x', record=True)
sound = StateMonitor(receptors, 'sound', record=0)

# Coincidence detectors
min_freq = 50 * Hz
max_freq = 1000 * Hz
N = 300
tau = 1 * ms
sigma = .1
eqs_neurons = '''
dv/dt=-v/tau+sigma*(2./tau)**.5*xi : 1
'''
neurons = NeuronGroup(N, model=eqs_neurons, threshold=1, reset=0)
synapses = Connection(receptors, neurons, 'v', structure='dense', max_delay=1.1 * max_delay, delay=True)
synapses.connect_full(receptors, neurons, weight=.5)
synapses.delay[1, :] = 1. / exp(linspace(log(min_freq / Hz), log(max_freq / Hz), N))
spikes = SpikeMonitor(neurons)

run(500 * ms)
raster_plot(spikes)
ylabel('Frequency')
yticks([0, 99, 199, 299], array(1. / synapses.delay.todense()[1, [0, 99, 199, 299]], dtype=int))
show()



from brian import *
from brian.stateupdater import get_linear_equations_solution_numerically, get_linear_equations
from scipy import linalg, weave
import numpy
import inspect
from itertools import count
import re

__all__ = ['MultiLinearStateUpdater', 'get_multilinear_state_updater', 'MultiLinearNeuronGroup']

__all__ = ['MultiLinearStateUpdater', 'get_multilinear_state_updater', 'MultiLinearNeuronGroup']

def get_multilinear_matrices(eqs, subs, clock=None):
    '''
    Returns the matrices M and B for the linear model dX/dt = M(X-B),
    where eqs is an Equations object. 
    '''
    if clock is None: clock = guess_clock()
    nsubs = len(subs[subs.keys()[0]])
    # Otherwise assumes it is given in functional form
    n = len(eqs._diffeq_names) # number of state variables
    dynamicvars = eqs._diffeq_names
    # Calculate B
    AB = zeros((n, 1, nsubs))
    d = dict.fromkeys(dynamicvars)
    for j in range(n):
        if dynamicvars[j] in subs:
            d[dynamicvars[j]] = subs[j]
        else:
            d[dynamicvars[j]] = 0. * eqs._units[dynamicvars[j]]
    for var, i in zip(dynamicvars, count()):
        AB[i, 0, :] = -eqs.apply(var, d)
    # Calculate A
    M = zeros((n, n, nsubs))
    for i in range(n):
        for j in range(n):
            if dynamicvars[j] in subs:
                d[dynamicvars[j]] = subs[j]
            else:
                d[dynamicvars[j]] = 0. * eqs._units[dynamicvars[j]]
        if dynamicvars[i] in subs:
            d[dynamicvars[i]] = subs[i]
        else:
            if isinstance(eqs._units[dynamicvars[i]], Quantity):
                d[dynamicvars[i]] = Quantity.with_dimensions(1., eqs._units[dynamicvars[i]].get_dimensions())
            else:
                d[dynamicvars[i]] = 1.
        for var, j in zip(dynamicvars, count()):
            M[j, i, :] = eqs.apply(var, d) + AB[j]

    allM = M
    allAB = AB

    AiBi = []
    for i in xrange(nsubs):
        try:
            M = reshape(allM[:, :, i], allM.shape[:2])
            AB = reshape(allAB[:, :, i], allAB.shape[:2])
            C = linalg.solve(M, AB)
            A = linalg.expm(M * clock.dt)
            B = -dot(A, C) + C
        except LinAlgError:
            numeulersteps = 100
            deltat = clock.dt / numeulersteps
            E = eye(n) + deltat * M
            C = eye(n)
            D = zeros((n, 1))
            for step in xrange(numeulersteps):
                C, D = dot(E, C), dot(E, D) - AB * deltat
            A, B = C, D
        AiBi.append((A, B))

    return AiBi

def get_multilinear_state_updater(eqs, subs, level=0, clock=None):
    '''
    Make a multilinear state updater
    
    Arguments:
    
    ``eqs``
        should be the equations, and must be a string not an :class:`Equations` object.
    ``subs``
        A dictionary of arrays, each key k is a variable name, each value is an equally
        sized array of values it takes, this array should have the size of the number
        of neurons in the group.
    ``level``
        How many levels up to look for the equations' namespace.
    ``clock``
        If you want.
    '''
    neweqs = Equations(eqs, level=level + 1)
    neweqs.prepare()
    AiBi = get_multilinear_matrices(neweqs, subs, clock=clock)
    n = AiBi[0][0].shape[0]
    nsubs = len(subs[subs.keys()[0]])
    A = numpy.zeros((n, n, nsubs))
    B = numpy.zeros((n, nsubs))
    for i, (Ai, Bi) in enumerate(AiBi):
        A[:, :, i] = Ai
        B[:, i] = Bi.squeeze()
    return MultiLinearStateUpdater(A, B)


class MultiLinearStateUpdater(StateUpdater):
    '''
    A StateUpdater with one differential equation for each neuron
    
    Initialise with:
    
    ``A``, ``B``
        Arrays, see below.
        
    The update step for neuron i is S[:,i] = dot(A[:,:,i], S[:,i]) + B[:, i]. 
    '''
    def __init__(self, A, B=None):
        self.A = A
        self.B = B
        self._useaccel = get_global_preference('useweave')
        self._cpp_compiler = get_global_preference('weavecompiler')
        self._extra_compile_args = ['-O3']
        if self._cpp_compiler == 'gcc':
            self._extra_compile_args += get_global_preference('gcc_options') # ['-march=native', '-ffast-math']

    def __call__(self, P):
        if self._useaccel:
            n = len(P)
            m = len(self)
            S = P._S
            A = self.A
            B = self.B
            code = '''
            double x[m]; 
            for(int i=0;i<n;i++)  
            {
                for(int j=0;j<m;j++)
                {
                    x[j] = B(j,i);
                    for(int k=0;k<m;k++)
                       x[j] += A(j,k,i) * S(k,i);
                }
                for(int j=0;j<m;j++) 
                    S(j,i) = x[j];
            } 
            '''
            weave.inline(code, ['n', 'm', 'S', 'A', 'B'],
                         compiler=self._cpp_compiler,
                         type_converters=weave.converters.blitz,
                         extra_compile_args=self._extra_compile_args)
        else:
            if self.A.shape[2] < self.A.shape[1]:
                for i in xrange(self.A.shape[2]):
                    P._S[:, i] = dot(self.A[:, :, i], P._S[:, i])
            else:
                # this is equivalent to the above but the loop is smaller if the
                # number of neurons is large
                AS = self.A[:, 0, :]*P._S[0, :]
                for i in xrange(1, self.A.shape[1]):
                    AS += self.A[:, i, :]*P._S[i, :]
                P._S[:] = AS
            if self.B is not None:
                add(P._S, self.B, P._S)

    def __len__(self):
        return self.A.shape[0]

    def __repr__(self):
        return 'MultiLinearStateUpdater'
    __str__ = __repr__


class MultiLinearNeuronGroup(NeuronGroup):
    '''
    Make a NeuronGroup with a linear differential equation for each neuron
    
    You give a single set of differential equations with parameters, the
    variables you want substituted should be defined as parameters in the equations,
    but they will not be treated as parameters, instead they will be substituted.
    You also pass a list of variables to have their values substituted, and these
    names should exist in the namespace initialising the MultiLinearNeuronGroup. 
    
    Arguments:
    
    ``eqs``
        should be the equations, and must be a string not an :class:`Equations` object.
    ``subs``
        A list of variables to be substituted with values.
    ``level``
        How many levels up to look for the equations' namespace.
    ``clock``
        If you want.
    ``kwds``
        Any additonal arguments to pass to :class:`NeuronGroup` init.
    '''
    #TODO: for consistency put the units in the equations, and multilinearneurongroup can extract them.
    def __init__(self, eqs, subs, clock=None, level=0, **kwds):
        subs_set = set(subs)
        param_pattern = re.compile('\s*(\w+)\s*:\s*(.*)')
        eqs = re.sub('\\\s*?\n', ' ', eqs) # compact multiline equations
        neweqs = ''
        subs = {}
        for line in eqs.splitlines():
            line = re.sub('#.*', '', line) # remove comments
            result = param_pattern.match(line)
            added = False
            if result is not None:
                name, unit = result.groups()
                if name in subs_set:
                    subs[name] = unit
                    added = True
            if not added:
                neweqs += line + '\n'
        eqs = neweqs
        frame = inspect.stack()[level + 1][0]
        ns_global, ns_local = frame.f_globals, frame.f_locals
        k0 = subs.keys()[0]
        if k0 in ns_local:
            nsubs = len(ns_local[k0])
        else:
            nsubs = len(ns_global[k0])
        neweqs = eqs
        for k, u in subs.iteritems():
            neweqs = neweqs.replace(k, '(0.*(' + u + '))')
        neweqs = Equations(neweqs, level=level + 1)
        NeuronGroup.__init__(self, nsubs, neweqs, clock=clock, **kwds)
        subs2 = {}
        for k, u in subs.iteritems():
            if k in ns_local:
                subs2[k] = ns_local[k]
            else:
                subs2[k] = ns_global[k]
        self._state_updater = get_multilinear_state_updater(eqs, subs2, level=level + 1, clock=clock)

if __name__ == '__main__':
    eqs = '''
    dv/dt = k*v/(1*second) : 1
    dw/dt = k*w/(1*second) : 1
    k : 1
    '''
    k = array([-1, -2, -3])
    subs = ['k']
    G = MultiLinearNeuronGroup(eqs, subs)
    G.v = 1
    G.w = 0
    M = StateMonitor(G, 'v', record=True)
    run(1 * second)
    for i in range(len(G)):
        plot(M.times, M[i])
    show()



'''
Two connected neurons with delays
'''
from brian import *
tau = 10 * ms
w = -1 * mV
v0 = 11 * mV
neurons = NeuronGroup(2, model='dv/dt=(v0-v)/tau : volt', threshold=10 * mV, reset=0 * mV, \
                    max_delay=5 * ms)
neurons.v = rand(2) * 10 * mV
W = Connection(neurons, neurons, 'v', delay=2 * ms)
W[0, 1] = w
W[1, 0] = w
S = StateMonitor(neurons, 'v', record=True)
#mymonitor=SpikeMonitor(neurons[0])
mymonitor = PopulationSpikeCounter(neurons)

run(500 * ms)
plot(S.times / ms, S[0] / mV)
plot(S.times / ms, S[1] / mV)
show()



'''
Analysis of voltage traces (current-clamp experiments or HH models).
'''
from brian import *
from numpy import *
#from brian.stdunits import mV
from brian.tools.io import *
from time import time
from scipy import optimize
from scipy import stats
from scipy.signal import lfilter

__all__ = ['fit_EIF', 'IV_curve', 'threshold_model', 'estimate_capacitance']

"""
TODO:
* A better fit_EIF function (conjugate gradient? Check also Badel et al., 2008. Or maybe just calculate gradient)
* Fit subthreshold kernel (least squares, see also Jolivet)
* Standard threshold methods
"""

"""
The good strategy:
* optimise for every tau for best threshold prediction
* find the tau that minimises misses
"""
def threshold_model(v, onsets=None, dt=1., tau=None):
    '''
    Fits adaptive threshold model.
    
    Model:
    tau*dvt/dt=vt0+a*[v-vi]^+
    
    tau can be fixed or found by optimization.
    
    Returns vt0, vi, a[, tau] (tau in timesteps by default)
    '''
    if onsets is None: onsets = spike_onsets(v)
    if tau is None:
        spikes = spike_mask(v, onsets)
        def f(tau):
            vt0, vi, a = threshold_model(v, onsets=onsets, dt=dt, tau=tau)
            theta = vt0 + a * lowpass(clip(v - vi, 0, inf), tau, dt=0.05)
            return mean(theta[-spikes] < v[-spikes]) # false alarm rate
        tau = float(optimize.fmin(lambda x:f(*x), 10., disp=0))
        return list(threshold_model(v, onsets=onsets, dt=dt, tau=tau)) + [tau]
    else:
        threshold = v[onsets]
        def f(vt0, vi, a, tau):
            return vt0 + a * lowpass(clip(v - vi, 0, inf), tau, dt=dt)
        return optimize.leastsq(lambda x:f(x[0], x[1], x[2], tau)[onsets] - threshold, [-55., -65., 1.])[0]

def estimate_capacitance(i, v, dt=1, guess=100.):
    '''
    Estimates capacitance from current-clamp recording
    with white noise (see Badel et al., 2008).
    
    Hint for units: if v and dt are in ms, then the capacitance has
    the same relationship to F than the current to A (pA -> pF).
    '''
    dv = diff(v) / dt
    i, v = i[:-1], v[:-1]
    mask = -spike_mask(v) # subthreshold trace
    dv, v, i = dv[mask], v[mask], i[mask]
    return optimize.fmin(lambda C:var(dv - i / C), guess, disp=0)[0]

def IV_curve(i, v, dt=1, C=None, bins=None, T=0):
    '''
    Dynamic I-V curve (see Badel et al., 2008)
    E[C*dV/dt-I]=f(V)
    T: time after spike onset to include in estimation.
    bins: bins for v (default: 20 bins between min and max).
    '''
    C = C or estimate_capacitance(i, v, dt)
    if bins is None: bins = linspace(min(v), max(v), 20)
    dv = diff(v) / dt
    v, i = v[:-1], i[:-1]
    mask = -spike_mask(v, spike_onsets(v) + T) # subthreshold trace
    dv, v, i = dv[mask], v[mask], i[mask]
    fv = i - C * dv # intrinsic current
    return array([mean(fv[(v >= vmin) & (v < vmax)]) for vmin, vmax in zip(bins[:-1], bins[1:])])

def fit_EIF(i, v, dt=1, C=None, T=0):
    '''
    Fits the exponential model of spike initiation in the
    phase plane (v,dv).
    T: time after spike onset to include in estimation.
    
    Returns gl, El, deltat, vt
    (leak conductance, leak reversal potential, slope factor, threshold)
    The result does not seem very reliable (deltat depends critically on T).
    
    Hint for units: if v is in mV, dt in ms, i in pA, then gl is in nS
    '''
    C = C or estimate_capacitance(i, v, dt)
    dv = diff(v) / dt
    v, i = v[:-1], i[:-1]
    mask = -spike_mask(v, spike_onsets(v) + T) # subthreshold trace
    dv, v, i = dv[mask], v[mask], i[mask]
    f = lambda gl, El, deltat, vt:C * dv - i - (gl * (El - v) + gl * deltat * exp((v - vt) / deltat))
    x, _ = optimize.leastsq(lambda x:f(*x), [50., -60., 3., -55.])
    return x

if __name__ == '__main__':
    filename = r"D:\My Dropbox\Neuron\Hu\recordings_Ifluct\I0_1_std_1_tau_10_sampling_20\vs.dat"
    #filename=r'D:\Anna\2010_03_0020_random_noise_200pA.atf'
    #filename2=r'D:\Anna\input_file.atf' # in pF
    from pylab import *
    #M=loadtxt(filename)
    #t,vs=M[:,0],M[:,1]
    t, vs = read_neuron_dat(filename)
    #t,vs=read_atf(filename)
    #print array(spike_duration(vs,full=True))*.05
    #vt0,vi,a,tau=threshold_model(vs,dt=0.05)
    #print vt0,vi,a,tau
    #theta=vt0+a*lowpass(clip(vs-vi,0,inf),tau,dt=0.05)
    spikes = spike_onsets(vs)
    print std(vs[spikes])
    #spikes2=spike_onsets_dv2(vs)
    #spikes3=spike_onsets_dv3(vs)
    plot(t, vs, 'k')
    #plot(t,theta,'b')
    plot(t[spikes], vs[spikes], '.r')
    #plot(t[spikes2],vs[spikes2],'.g')
    #plot(t[spikes3],vs[spikes3],'.k')
    show()



from ...stateupdater import StateUpdater
from ...globalprefs import get_global_preference
from ...clock import guess_clock
from ...log import log_debug, log_warn
from codegen_c import *
from codegen_python import *
from integration_schemes import *
import time
from scipy import weave
import numpy, scipy
import re
from c_support_code import *
try:
    import numexpr as numexpr
except ImportError:
    numexpr = None

__all__ = ['CStateUpdater', 'PythonStateUpdater']


class CStateUpdater(StateUpdater):
    def __init__(self, eqs, scheme, clock=None, freeze=False):
        self.clock = guess_clock(clock)
        self.eqs = eqs
        self.scheme = scheme
        self.freeze = freeze
        self.code_c = CCodeGenerator().generate(eqs, scheme)
        log_debug('brian.experimental.codegen.stateupdaters', 'C state updater code:\n' + self.code_c)
        self._weave_compiler = get_global_preference('weavecompiler')
        self._extra_compile_args = ['-O3']
        if self._weave_compiler == 'gcc':
            self._extra_compile_args += get_global_preference('gcc_options') # ['-march=native', '-ffast-math']
        self.namespace = {}
        code_vars = re.findall(r'\b\w+\b', self.code_c)
        self._arrays_to_check = []
        for varname in code_vars:
            if varname not in eqs._eq_names + eqs._diffeq_names + eqs._alias.keys() + ['t', 'dt', '_S', 'num_neurons']:
                # this is kind of a hack, but since we're going to be writing a new
                # and more sensible Equations module anyway, it can stand
                for name in eqs._namespace:
                    if varname in eqs._namespace[name]:
                        varval = eqs._namespace[name][varname]
                        if isinstance(varval, numpy.ndarray):
                            self.namespace[varname] = varval
                            self._arrays_to_check.append((varname, varval))
                            self.code_c = re.sub(r'\b'+varname+r'\b', varname+'[_i]', self.code_c)
                            break

    def __call__(self, P):
        if self._arrays_to_check is not None:
            N = len(P)
            for name, X in self._arrays_to_check:
                if len(X)!=N:
                    raise ValueError('Array '+name+' has wrong size ('+str(len(X))+' instead of '+str(N)+')')
            self._arrays_to_check = None
        self.namespace['dt'] = P.clock._dt
        self.namespace['t'] = P.clock._t
        self.namespace['num_neurons'] = len(P)
        self.namespace['_S'] = P._S
        try:
            weave.inline(self.code_c, self.namespace.keys(),#['_S', 'num_neurons', 'dt', 't'],
                         local_dict = self.namespace,
                         support_code=c_support_code,
                         compiler=self._weave_compiler,
                         extra_compile_args=self._extra_compile_args)
        except:
            log_warn('brian.experimental.codegen.stateupdaters',
                     'C compilation failed, falling back on Python.')
            self.__class__ = PythonStateUpdater
            self.__init__(self.eqs, self.scheme, self.clock, self.freeze)
            self.__call__(P)


class PythonStateUpdater(StateUpdater):
    def __init__(self, eqs, scheme, clock=None, freeze=False):
        eqs.prepare()
        self.clock = guess_clock(clock)
        self.code_python = PythonCodeGenerator().generate(eqs, scheme)
        self.compiled_code = compile(self.code_python, 'StateUpdater code', 'exec')
        if False and numexpr is not None:
            # This only improves things for large N, in which case Python speed
            # is close to C speed anyway, so less valuable
            newcode = ''
            for line in self.code_python.split('\n'):
                m = re.search(r'(\b\w*\b\s*[^><=]?=\s*)(.*)', line) # lines of the form w = ..., w *= ..., etc.
                if m:
                    if '*' in m.group(2) or '+' in m.group(2) or '/' in m.group(2) or \
                       '-' in m.group(2) or '**' in m.group(2) or '(' in m.group(2):
                        if '[' not in m.group(2):
                            line = m.group(1) + "_numexpr.evaluate('" + m.group(2) + "')"
                newcode += line + '\n'
            self.code_python = newcode
        self.code_python = 'def _stateupdate(_S, dt, t, num_neurons):\n' + '\n'.join(['    ' + line for line in self.code_python.split('\n')]) + '\n'
        self.namespace = {'_numexpr':numexpr}
        for varname in self.compiled_code.co_names:
            if varname not in eqs._eq_names + eqs._diffeq_names + eqs._alias.keys() + ['t', 'dt', '_S', 'num_neurons']:
                # this is kind of a hack, but since we're going to be writing a new
                # and more sensible Equations module anyway, it can stand
                for name in eqs._namespace:
                    if varname in eqs._namespace[name]:
                        self.namespace[varname] = eqs._namespace[name][varname]
                        break
                if varname not in self.namespace:
                    if hasattr(numpy, varname):
                        self.namespace[varname] = getattr(numpy, varname)
        log_debug('brian.experimental.codegen.stateupdaters', 'Python state updater code:\n' + self.code_python)
        exec self.code_python in self.namespace
        self.state_update_func = self.namespace['_stateupdate']

    def __call__(self, P):
        self.state_update_func(P._S, P.clock._dt, P.clock._t, len(P))

if __name__ == '__main__':
    from brian import *

    duration = 1 * second
    N = 10000
    domonitor = False

#    duration = 100*ms
#    N = 10
#    domonitor = True    

    eqs = Equations('''
    #dV/dt = -V*V/(10*ms) : 1
    #dV/dt = cos(2*pi*t/(100*ms))/(10*ms) : 1
    dV/dt = -W*V/(100*ms) : 1
    dW/dt = -W/(100*ms) : 1
    #dV/dt = cos(2*pi*t/(100*ms))/(10*ms) : 1
    #dW/dt = cos(2*pi*t/(100*ms))/(10*ms) : 1
    #dW2/dt = cos(2*pi*t/(100*ms))/(10*ms) : 1
    #dV/dt = h/(10*ms) : 1
    #h = -V*V : 1
    ''')
    print eqs

    G = NeuronGroup(N, eqs, compile=True, freeze=True, implicit=True)

    print G._state_updater.__class__

#    su = PythonStateUpdater(eqs, euler_scheme, clock=G.clock)
#    print 'Python code:'
#    print su.code_python
    su = CStateUpdater(eqs, exp_euler_scheme, clock=G.clock)
    print 'C++ loop code:'
    print su.code_c

    G.V = 1
    G.W = 0.1

    if domonitor:
        M = StateMonitor(G, 'V', record=True)

    S = copy(G._S)
    run(1 * ms)
    start = time.time()
    run(duration)
    print 'Original code:', (time.time() - start) * second
    if domonitor: M_V = M[0]
    reinit()
    G._S[:] = S
    G._state_updater = su
    run(1 * ms)
    start = time.time()
    run(duration)
    print 'New code:', (time.time() - start) * second
    if domonitor: M_V2 = M[0]

    if domonitor:
        print amax(abs(M_V - M_V2))
        plot(M.times, M_V)
        plot(M.times, M_V2)
        show()



import brian_no_units
from brian import *
from filter import *
from scipy.optimize import *

class ElectrodeCompensation (object):
    eqs = """
            dV/dt=Re*(-Iinj)/taue : volt
            dV0/dt=(R*Iinj-V0+Vr)/tau : volt
            Iinj=(V-V0)/Re : amp
            """

    def __init__(self, I, Vraw,
                 dt=defaultclock.dt, durslice=1*second,
                  p=1.0, 
                 *params):
        self.I = I
        self.Vraw = Vraw
        self.p = p
        self.dt = dt
        self.x0 = self.params_to_vector(*params)
        self.duration = len(I) * dt
        self.durslice = min(durslice, self.duration)
        self.slicesteps = int(durslice/dt)
        self.nslices = int(ceil(len(I)*dt/durslice))
        
        self.islice = 0
        self.I_list = [I[self.slicesteps*i:self.slicesteps*(i+1)] for i in range(self.nslices)]
        self.Vraw_list = [Vraw[self.slicesteps*i:self.slicesteps*(i+1)] for i in range(self.nslices)]

    def vector_to_params(self, *x):
        R,tau,Vr,Re,taue = x

        R = R*R
        tau = tau*tau
        Re = Re*Re
        taue = taue*taue

        return R,tau,Vr,Re,taue

    def params_to_vector(self, *params):
        x = params

        x = [sqrt(params[0]),
             sqrt(params[1]),
             params[2],
             sqrt(params[3]),
             sqrt(params[4])]

        return list(x)

    def get_model_trace(self, row, *x):
        R, tau, Vr, Re, taue = self.vector_to_params(*x)
        eqs = Equations(self.eqs)
        eqs.prepare()
        self._eqs = eqs
        y = simulate(eqs, self.I_list[self.islice] * Re/taue, self.dt, row=row)
        return y

    def fitness(self, x):
        R, tau, Vr,  Re, taue = self.vector_to_params(*x)
        y = self.get_model_trace(0, *x)
        e = self.dt*sum(abs(self.Vraw_list[self.islice]-y)**self.p)
        return e

    def compensate_slice(self, x0):
        fun = lambda x: self.fitness(x)
        x = fmin(fun, x0, maxiter=10000, maxfun=10000)
        return x

    def compensate(self):
        self.params_list = []
        self.xlist = [self.x0]
        t0 = time.clock()
        for self.islice in range(self.nslices):
            newx = self.compensate_slice(self.xlist[self.islice])
            self.xlist.append(newx)
            self.params_list.append(self.vector_to_params(*newx))
            print "Slice %d/%d compensated in %.2f seconds" %  \
                (self.islice+1, self.nslices, time.clock()-t0)
            t0 = time.clock()
        self.xlist = self.xlist[1:]
        return self.xlist

    def get_compensated_trace(self):
        Vcomp_list = []
        Vmodel_list = []
        for self.islice in range(self.nslices):
            x = self.xlist[self.islice]
            V = self.get_model_trace(0, *x)
            V0 = self.get_model_trace(1, *x)
            Velec = V-V0
            Vcomp_list.append(self.Vraw_list[self.islice] - Velec)
            Vmodel_list.append(V)
        self.Vcomp = hstack(Vcomp_list)
        self.Vmodel = hstack(Vmodel_list)
        return self.Vcomp, self.Vmodel

def compensate(I, Vraw, 
               dt=defaultclock.dt, 
               durslice=1*second,
               p=1.0,
               **initial_params):

    R = initial_params.get("R", 100*Mohm)
    tau = initial_params.get("tau", 20*ms)
    Vr = initial_params.get("Vr", -70*mV)
    Re = initial_params.get("Re", 50*Mohm)
    taue = initial_params.get("taue", .5*ms)

    comp = ElectrodeCompensation(I, Vraw,
                                 dt,
                                 durslice,
                                 p,
                                 R, tau, Vr, Re, taue)
    comp.compensate()
    Vcomp, Vmodel = comp.get_compensated_trace()
    params = comp.params_list
    return Vcomp, Vmodel, params


if __name__ == '__main__':

    I = numpy.load("current1.npy")[:100000] * 80
    Vraw = numpy.load("trace1.npy")[:100000]

    R = 200*Mohm
    tau = 20*ms
    Vr = -70*mV
    Re = 50*Mohm
    taue = .5*ms

    Vcomp, Vmodel, params = compensate(I, Vraw, dt=.1*ms,
                                       p = 1.0,
                                        durslice=10*second,
                                        R=R, tau=tau, Vr=Vr, Re=Re, taue=taue)

    #subplot(211)
    #plot(I)

    #subplot(212)
    plot(Vraw, 'k')
    plot(Vmodel, 'g')
    plot(Vcomp, 'r')

    show()



'''
Example showing :func:`linked_var`, connecting two different :class:`NeuronGroup`
variables. Here we show something like a simplified haircell and auditory nerve
fibre model where the hair cells and ANFs are implemented as two separate
:class:`NeuronGroup` objects. The hair cells filter their inputs via a 
differential equation, and then emit graded amounts of neurotransmitter
(variable 'y') to the auditory nerve fibres input current (variable 'I').
'''
from brian import *

N = 5
f = 50 * Hz
a_min = 1.0
a_max = 100.0
tau_haircell = 50 * ms
tau = 10 * ms
duration = 100 * ms

eqs_haircells = '''
input = a*sin(2*pi*f*t) : 1
x = clip(input, 0, Inf)**(1.0/3.0) : 1
a : 1
dy/dt = (x-y)/tau_haircell : 1 
'''

haircells = NeuronGroup(N, eqs_haircells)
haircells.a = linspace(a_min, a_max, N)
M_haircells = MultiStateMonitor(haircells, vars=('input', 'y'), record=True)

eqs_nervefibres = '''
dV/dt = (I-V)/tau : 1
I : 1
'''
nervefibres = NeuronGroup(N, eqs_nervefibres, reset=0, threshold=1)
nervefibres.I = linked_var(haircells, 'y')
M_nervefibres = MultiStateMonitor(nervefibres, record=True)

run(duration)

subplot(221)
M_haircells['input'].plot()
ylabel('haircell.input')
subplot(222)
M_haircells['y'].plot()
ylabel('haircell.y')
subplot(223)
M_nervefibres['I'].plot()
ylabel('nervefibres.I')
subplot(224)
M_nervefibres['V'].plot()
ylabel('nervefibres.V')
show()



from scipy import sparse
import numpy
from itertools import izip
from operator import isSequenceType
import bisect
from numpy import ndarray
import itertools


class lil_matrix(sparse.lil_matrix):
    # Unfortunately we still need to implement this because although scipy 0.7.0
    # now supports X[a:b,c:d] for sparse X it is unbelievably slow (shabby code
    # on their part).
    def __setitem__(self, index, W):
        """
        Speed-up if x is a sparse matrix.
        TODO: checks (first remove the data).
        TODO: once we've got this working in all cases, should we submit to scipy?
        """
        try:
            i, j = index
        except (ValueError, TypeError):
            raise IndexError, "invalid index"

        if isinstance(i, slice) and isinstance(j, slice) and\
               (i.step is None) and (j.step is None) and\
               (isinstance(W, sparse.lil_matrix) or isinstance(W, numpy.ndarray)):
            rows = self.rows[i]
            datas = self.data[i]
            j0 = j.start
            if isinstance(W, sparse.lil_matrix):
                for row, data, rowW, dataW in izip(rows, datas, W.rows, W.data):
                    jj = bisect.bisect(row, j0) # Find the insertion point
                    row[jj:jj] = [j0 + k for k in rowW]
                    data[jj:jj] = dataW
            elif isinstance(W, ndarray):
                nq = W.shape[1]
                for row, data, rowW in izip(rows, datas, W):
                    jj = bisect.bisect(row, j0) # Find the insertion point
                    row[jj:jj] = range(j0, j0 + nq)
                    data[jj:jj] = rowW
        elif isinstance(i, int) and isinstance(j, (list, tuple, numpy.ndarray)):
            row = dict(izip(self.rows[i], self.data[i]))
            try:
                row.update(dict(izip(j, W)))
            except TypeError:
                row.update(dict(izip(j, itertools.repeat(W))))
            items = row.items()
            items.sort()
            row, data = izip(*items)
            self.rows[i] = list(row)
            self.data[i] = list(data)
        elif isinstance(i, slice) and isinstance(j, int) and isSequenceType(W):
            # This corrects a bug in scipy sparse matrix as of version 0.7.0, but
            # it is not efficient!
            for w, k in izip(W, xrange(*i.indices(self.shape[0]))):
                sparse.lil_matrix.__setitem__(self, (k, j), w)
        else:
            sparse.lil_matrix.__setitem__(self, index, W)



from brian import *
import multiprocessing
from numpy import ctypeslib
import ctypes
from multiprocessing import sharedctypes
import numpy

def worker(S, shape, i, j, A, C, conn):
    S = ctypeslib.as_array(S)
    S.shape = shape
    S = S[:, i:j]
    sys.stdout.flush()
    while True:
        try:
            job = conn.recv()
        except EOFError:
            job = None
        if job is None:
            break
        S[:] = dot(A, S)
        if C is not None:
            add(S, C, S)
        conn.send(True)
    conn.close()


class MulticoreLinearStateUpdater(LinearStateUpdater):
    def __init__(self, stateupdater, group, ncpu=None):
        self.stateupdater = stateupdater
        self.group = group
        if ncpu is None:
            ncpu = multiprocessing.cpu_count()
        self.ncpu = ncpu
        subgroup_sizes = [len(group) / ncpu] * ncpu
        subgroup_sizes[-1] += len(group) - sum(subgroup_sizes)
        I = hstack((0, cumsum(subgroup_sizes)))
        S = group._S
        shape = S.shape
        S = reshape(S, S.size)
        size = S.size
        S = sharedctypes.RawArray('d', S)
        # must replace group._S to use this shared memory!
        group._S = numpy.frombuffer(S, dtype=numpy.float64, count=size)
        group._S.shape = shape
        A = stateupdater.A
        if stateupdater._useB:
            C = stateupdater._C
        else:
            C = None
        self.pipes = [multiprocessing.Pipe() for _ in xrange(ncpu)]
        self.server_conns, self.client_conns = zip(*self.pipes)
        self.processes = [multiprocessing.Process(
                                target=worker,
                                args=(S, shape, i, j, A, C, conn)
                                ) for i, j, conn in zip(I[:-1],
                                                        I[1:],
                                                        self.client_conns)]
        for p in self.processes:
            p.start()

    def __call__(self, G):
        assert G is self.group
        for conn in self.server_conns:
            conn.send(True)
        return [conn.recv() for conn in self.server_conns]

    def __del__(self):
        for conn in self.server_conns:
            conn.send(None)
        for p in self.processes:
            p.terminate()

if __name__ == '__main__':
    import time

    # timings on my dual-core Dell E4300 for N=40,000
    # with connections:
    # ncpu=1: 18.5
    # ncpu=2: 12.5
    # without connections:
    # ncpu=1: 17.5
    # ncpu=2: 12.9
    # without connections for N=400,000 and duration=.01*second:
    # ncpu=1: 28.1
    # ncpu=2: 31.2
    # without connections N=4,000 and duration=10*second:
    # ncpu=1: 1.6s
    # ncpu=2: 1.9s

    ncpu = None
    duration = 1 * second
    N = 4000
    monitor = True
    have_connections = True

    Ni = int(N * .2)
    Ne = N - Ni
    p = 80.0 / N

    eqs = '''
    dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
    dge/dt = -ge/(5*ms) : volt
    dgi/dt = -gi/(10*ms) : volt
    '''

    P = NeuronGroup(N, model=eqs,
                  threshold= -50 * mV, reset= -60 * mV)
    P.v = -60 * mV + 10 * mV * rand(len(P))
    Pe = P.subgroup(Ne)
    Pi = P.subgroup(Ni)

    if have_connections:
        Ce = Connection(Pe, P, 'ge')
        Ci = Connection(Pi, P, 'gi')
        Ce.connect_random(Pe, P, p, weight=1.62 * mV)
        Ci.connect_random(Pi, P, p, weight= -9 * mV)

    if monitor:
        M = SpikeMonitor(P)
        Mv = StateMonitor(P, 'v', record=[0])

    if ncpu is None or ncpu > 1:
        P._state_updater = MulticoreLinearStateUpdater(P._state_updater, P,
                                                       ncpu=ncpu)

    run(1 * msecond)

    start = time.time()

    run(duration)

    end = time.time()

    print 'N:', N
    print 'duration:', duration
    print 'ncpu:', ncpu
    print 'Time taken:', end - start

    del P._state_updater

    if monitor:
        subplot(211)
        raster_plot(M)
        subplot(212)
        Mv.plot()
        show()



from filterbank import *
from linearfilterbank import *
from firfilterbank import *
from filterbanklibrary import *
from filterbankgroup import *
from drnl import DRNL
from dcgc import DCGC
from fractionaldelay import *
#from zilany import ZILANY
from tan_carney import TanCarney


from brian import *
from brian.library.random_processes import *
import multiprocessing

# Parameters
C = 281 * pF
gL = 30 * nS
taum = C / gL
EL = -70.6 * mV
VT = -50.4 * mV
DeltaT = 2 * mV
Vcut = VT + 5 * DeltaT
taue = 2.728 * ms
taui = 10.49 * ms
Ee = 0 * mV
Ei = -75 * mV

eGain = 1
iGain = 2
# Pick an electrophysiological behaviour
tauw, a, b, Vr = 144 * ms, 4 * nS, 0.0805 * nA, -70.6 * mV # Regular spiking (as inthe paper)
#tauw,a,b,Vr=20*ms,4*nS,0.5*nA,VT+5*mV # Bursting
#tauw,a,b,Vr=144*ms,2*C/(144*ms),0*nA,-70.6*mV # Fast spiking

eqs = """
dvm/dt=(gL*(EL-vm)+gL*DeltaT*exp((vm-VT)/DeltaT)+I-w+eGain*ge*(Ee-vm)+iGain*gi*(Ei-vm))/C  : volt
dw/dt=(a*(vm-EL)-w)/tauw : amp
I : amp
"""
eqs += OrnsteinUhlenbeck('ge', mu=2. * gL / 3, sigma=2. * gL / 3, tau=taue)
eqs += OrnsteinUhlenbeck('gi', mu=4. * gL / 3, sigma=4. * gL / 3, tau=taui)

"""Trace is a non-dimensionalized array of voltages a StateMonitor
recorded. Spikes is the accompanying SpikeMonitor passed to ease
finding spikes in trace. Cutoff is the voltage at which to truncate
each spike."""
def formatTraces(trace, spikes, cutoff, dt):
    answer = trace
    for _, t in spikes.spikes:answer[int(t / defaultclock.dt)] = 20 * mV
    return answer

# This is the function that we want to compute for various different parameters
def singleNeuronTrial(duration):
    # These two lines reset the clock to 0 and clear any remaining data so that
    # memory use doesn't build up over multiple runs.
    reinit_default_clock()
    clear(True)
    neuron = NeuronGroup(1, model=eqs, threshold=Vcut, reset="vm=Vr;w+=b", freeze=True)
    neuron.vm = EL
    trace = StateMonitor(neuron, 'vm', record=True)
    cutoff = 20 * mV
    spikes = SpikeMonitor(neuron)
    excitation = StateMonitor(neuron, 'ge', record=True)
    inhibition = StateMonitor(neuron, 'gi', record=True)
    run(duration * ms)
    formattedTrace = format(trace[0], spikes, cutoff, defaultclock.dt)
    return formattedTrace, trace.times / ms, excitation, inhibition

def visualize(results):

    #Compute Averages
    traces = mean(results[0])
    times = results[1]
    excitations = mean(results[2])
    exTimes = results[2].times
    inhibitions = mean(results[3])
    inTimes = results[3].times

    figure()
    subplot(211)
    title('Average AdExp Basal Activity over 100 2s Trial')
    ylabel('Voltage (mV)')
    plot(times / ms, traces / mV)
    subplot(212)
    plot(exTimes / ms, (excitations * (Ee - traces)) / nA)
    plot(Intimes / ms, (inhibitions * (Ei - traces)) / nA, 'r')
    title('Background Activity')
    ylabel('Current (nA)')
    xlabel('Time (ms)')
    legend(('Excitation', 'Inihibition'))

if __name__ == '__main__':
    pool = multiprocessing.Pool() # uses num_cpu processes by default
    runNumber = 10
    duration = 20
    args = [duration] * runNumber
    #results = pool.map(singleNeuronTrial, args)
    results = map(singleNeuronTrial, args)
    visualize(results)
    show()



import os
os.chdir('../../../docs_sphinx') # work from docs_sphinx/ directory until we make the big change
# if this doesn't work, run build_html_clean.py instead
os.system('sphinx-build -a -E -D building_as=latex -b latex . ./_latexbuild')
os.chdir('_latexbuild')
os.system('make')



from time import time
#http://amtoolbox.sourceforge.net/doc/filters/gammatone.php
from brian import *
set_global_preferences(useweave=True)
from scipy.io import loadmat,savemat

from brian.hears import *
#from zilany import *


simulation_duration = 1*ms
set_default_samplerate(100*kHz)
#sound = whitenoise(simulation_duration)
file="/home/bertrand/Data/MatlabProg/brian_hears/ZilanyCarney-JASAcode-2009/sound.mat"
X=loadmat(file,struct_as_record=False)
sound = Sound(X['sound'].flatten())
sound.samplerate = 100*kHz
#sound = sound.atlevel(10*dB) # level in rms dB SPL
#X={}
#X['sound'] = sound.__array__()
#savemat('/home/bertrand/Data/MatlabProg/brian_hears/ZilanyCarney-JASAcode-2009/sound.mat',X)

#sound = Sound(randn(1000))
#plot(sound)
#show()
#sound.samplerate = 100*kHz
cf = array([100*Hz,100*Hz,100*Hz,100*Hz])#erbspace(100*Hz, 1000*Hz, 50) # centre frequencies
cf = erbspace(100*Hz, 1000*Hz, 500) # centre frequencies

param_drnl = {}
#param_drnl['lp_nl_cutoff_m'] = 1.1
zilany_filter=ZILANY(sound, cf,32)
#zilany_filter=DRNL(sound, cf)
t1=time()
drnl = zilany_filter.process()
print time()-t1
#drnl =zilany_filter.rsigma
X={}
X['out_BM'] = drnl[:]
#X['out_BM'] = zilany_filter.rsigma
savemat('/home/bertrand/Data/MatlabProg/brian_hears/ZilanyCarney-JASAcode-2009/out_BM.mat',X)

#figure()
subplot(211)
##print drnl[:]R
plot(drnl[:])

#imshow(flipud(drnl.T), aspect='auto')
subplot(212)
#print sound
plot(sound)
#imshow(flipud(dcgc.T), aspect='auto')
show()



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
"""Direct controlling mechanisms

NeuronGroups and callable objects which allow direct
control over the behaviour of neurons.
"""

__all__ = ['MultipleSpikeGeneratorGroup', 'SpikeGeneratorGroup', 'PulsePacket',
           'PoissonGroup', 'OfflinePoissonGroup', 'PoissonInput']

from neurongroup import *
from threshold import *
from stateupdater import *
from units import *
import random as pyrandom
from numpy import where, array, zeros, ones, inf, nonzero, tile, sum, isscalar,\
                  cumsum, hstack, bincount,  ceil, ndarray, ascontiguousarray
from copy import copy
from clock import guess_clock
from utils.approximatecomparisons import *
import warnings
from operator import itemgetter
from log import *
import numpy
from numpy.random import exponential, randint, binomial
from connections import Connection

class MultipleSpikeGeneratorGroup(NeuronGroup):
    """Emits spikes at given times
    
    **Initialised as:** ::
    
        MultipleSpikeGeneratorGroup(spiketimes[,clock[,period]])
    
    with arguments: 

    ``spiketimes``
        a list of spike time containers, one for each neuron in the group,
        although note that elements of ``spiketimes`` can also be callable objects which
        return spike time containers if you want to be able to reinitialise (see below).
        At it's simplest, ``spiketimes`` could be a list of lists, where ``spiketimes[0]`` contains
        the firing times for neuron 0, ``spiketimes[1]`` for neuron 1, etc. But, any iterable
        object can be passed, so ``spiketimes[0]`` could be a generator for example. Each
        spike time container should be sorted in time. If the containers are numpy arrays units
        will not be checked (times should be in seconds).
    ``clock``
        A clock, if omitted the default clock will be used.
    ``period``
        Optionally makes the spikes recur periodically with the given
        period. Note that iterator objects cannot be used as the ``spikelist``
        with a period as they cannot be reinitialised.
    
    Note that if two or more spike times fall within the same ``dt``, spikes will stack up
    and come out one per dt until the stack is exhausted. A warning will be generated
    if this happens.

    Also note that if you pass a generator, then reinitialising the group will not have the
    expected effect because a generator object cannot be reinitialised. Instead, you should
    pass a callable object which returns a generator, this will be called each time the
    object is reinitialised by calling the ``reinit()`` method.
        
    **Sample usage:** ::
    
        spiketimes = [[1*msecond, 2*msecond]]
        P = MultipleSpikeGeneratorGroup(spiketimes)
    """
    def __init__(self, spiketimes, clock=None, period=None):
        """Pass spiketimes
        
        spiketimes is a list of lists, one list for each neuron
        in the group. Each sublist consists of the spike times.
        """
        clock = guess_clock(clock)
        thresh = MultipleSpikeGeneratorThreshold(spiketimes, period=period)
        NeuronGroup.__init__(self, len(spiketimes), model=LazyStateUpdater(), threshold=thresh, clock=clock)

    def reinit(self):
        super(MultipleSpikeGeneratorGroup, self).reinit()
        self._threshold.reinit()

    def __repr__(self):
        return "MultipleSpikeGeneratorGroup"


class MultipleSpikeGeneratorThreshold(Threshold):
    def __init__(self, spiketimes, period=None):
        self.set_spike_times(spiketimes, period=period)

    def reinit(self):
        # spiketimes is a container where each element is an iterable container of spike times for
        # each neuron. We store the iterator for each neuron, and the next spike time if it exists
        # or None if there are no spikes or no more spikes
        def makeiter(obj):
            if callable(obj): return iter(obj())
            return iter(obj)
        self.spiketimeiter = [makeiter(st) for st in self.spiketimes]
        self.nextspiketime = [None for st in self.spiketimes]
        for i in range(len(self.spiketimes)):
            try:
                self.nextspiketime[i] = self.spiketimeiter[i].next()
            except StopIteration:
                pass
        self.curperiod = -1

    def set_spike_times(self, spiketimes, period=None):
        self.spiketimes = spiketimes
        self.period = period
        self.reinit()

    def __call__(self, P):
        firing = zeros(len(self.spiketimes))
        t = P.clock.t
        if self.period is not None:
            cp = int(t / self.period)
            if cp > self.curperiod:
                self.reinit()
                self.curperiod = cp
            t = t - cp * self.period
        # it is the iterator for neuron i, and nextspiketime is the stored time of the next spike
        for it, nextspiketime, i in zip(self.spiketimeiter, self.nextspiketime, range(len(self.spiketimes))):
            # Note we use approximate equality testing because the clock t+=dt mechanism accumulates errors
            if isinstance(self.spiketimes[i], numpy.ndarray):
                curt = float(t)
            else:
                curt = t
            if nextspiketime is not None and is_approx_less_than_or_equal(nextspiketime, curt):
                firing[i] = 1
                try:
                    nextspiketime = it.next()
                    if is_approx_less_than_or_equal(nextspiketime, curt):
                        log_warn('brian.MultipleSpikeGeneratorThreshold', 'Stacking multiple firing times')
                except StopIteration:
                    nextspiketime = None
                self.nextspiketime[i] = nextspiketime
        return where(firing)[0]


class SpikeGeneratorGroup(NeuronGroup):
    """Emits spikes at given times
    
    Initialised as::
    
        SpikeGeneratorGroup(N,spiketimes[,clock[,period]])
    
    with arguments:
    
    ``N``
        The number of neurons in the group.
    ``spiketimes``
        An object specifying which neurons should fire and when. It can be a container
        such as a ``list``, containing tuples ``(i,t)`` meaning neuron ``i`` fires at
        time ``t``, or a callable object which returns such a container (which
        allows you to use generator objects even though this is slower, see below). ``i`` can be an integer
        or an array (list of neurons that spike at the same time).
        If ``spiketimes`` is not a list or tuple, the pairs ``(i,t)`` need to be
        sorted in time. You can also pass a numpy array
        ``spiketimes`` where the first column of the array
        is the neuron indices, and the second column is the times in
        seconds. Alternatively you can pass a tuple with two arrays, the first one being the neuron indices and the second one times. WARNING: units are not checked in this case, the time array should be in seconds.
    ``clock``
        An optional clock to update with (omit to use the default clock).
    ``period``
        Optionally makes the spikes recur periodically with the given
        period. Note that iterator objects cannot be used as the ``spikelist``
        with a period as they cannot be reinitialised.
    ``gather=False``
        Set to True if you want to gather spike events that fall in the same
        timestep. (Deprecated since Brian 1.3.1)
    ``sort=True``
        Set to False if your spike events are already sorted.
    
    Has an attribute:
    
    ``spiketimes``
        This can be used to reset the list of spike times, however the values of
        ``N``, ``clock`` and ``period`` cannot be changed. 
        
    **Sample usages**
    
    The simplest usage would be a list of pairs ``(i,t)``::
    
        spiketimes = [(0,1*ms), (1,2*ms)]
        SpikeGeneratorGroup(N,spiketimes)
    
    A more complicated example would be to pass a generator::

        import random
        def nextspike():
            nexttime = random.uniform(0*ms,10*ms)
            while True:
                yield (random.randint(0,9),nexttime)
                nexttime = nexttime + random.uniform(0*ms,10*ms)
        P = SpikeGeneratorGroup(10,nextspike())
    
    This would give a neuron group ``P`` with 10 neurons, where a random one
    of the neurons fires at an average rate of one every 5ms. Please note that as of 1.3.1, this behavior is preserved but will run slower than initializing with arrays, or lists.
    
    **Notes**
    
    Note that if a neuron fires more than one spike in a given interval ``dt``, additional
    spikes will be discarded. If you want them to stack, consider using the less efficient
    :class:`MultipleSpikeGeneratorGroup` object instead. A warning will be issued if this
    is detected.

    Also, if you want to use a SpikeGeneratorGroup with many spikes and/or neurons, please use an initialization with arrays.
    
    Also note that if you pass a generator, then reinitialising the group will not have the
    expected effect because a generator object cannot be reinitialised. Instead, you should
    pass a callable object which returns a generator. In the example above, that would be
    done by calling::
    
        P = SpikeGeneratorGroup(10,nextspike)
        
    Whenever P is reinitialised, it will call ``nextspike()`` to create the required spike
    container.
    """
    def __init__(self, N, spiketimes, clock=None, period=None, 
                 sort=True, gather=None):
        clock = guess_clock(clock)
        self.N = N
        self.period = period
        if gather:
            log_warn('brian.SpikeGeneratorGroup', 'SpikeGeneratorGroup\'s gather keyword use is deprecated')
        fallback = False # fall back on old SpikeGeneratorThreshold or not
        if isinstance(spiketimes, list):
            # spiketimes is a list of (i,t)
            if len(spiketimes):
                idx, times = zip(*spiketimes)
            else:
                idx, times = [], []
            # the following try ... handles the case where spiketimes has index arrays
            # e.g spiketimes = [([0, 1], 0 * msecond), ([0, 1, 2], 2 * msecond)]
            # Notes:
            # - if there is always the same number of indices by array, its simple, it's just a matter of flattening
            # - if not, then it requires a for loop, and it's done in the except
            try:
                idx = array(idx, dtype = float)
                times = array(times, dtype = float)
                if idx.ndim > 1:
                    # simple case
                    times = tile(times.reshape((len(times), 1)), (idx.shape[1], 1)).flatten()
                    idx = idx.flatten()
            except ValueError:
                new_idx = []
                new_times = []
                for k, item in enumerate(idx):
                    if isinstance(item, list):
                        new_idx += item # append indices
                        new_times += [times[k]]*len(item)
                    else:
                        new_times += [times[k]]
                        new_idx += [item]
                idx = array(new_idx, dtype  = float)
                times = new_times
                times = array(times, dtype = float)
        elif isinstance(spiketimes, tuple):
            # spike times is a tuple with idx, times in arrays
            idx = spiketimes[0]
            times = spiketimes[1]
        elif isinstance(spiketimes, ndarray):
            # spiketimes is a ndarray, with first col is index and second time
            idx = spiketimes[:,0]
            times = spiketimes[:,1]
        else:
            log_warn('brian.SpikeGeneratorGroup', 'Using (slow) threshold because spiketimes is assumed to be a generator/iterator')
            # spiketimes is a callable object, so falling back on old SpikeGeneratorThreshold
            fallback = True

        if not fallback:
            thresh = FastSpikeGeneratorThreshold(N, idx, times, dt=clock.dt, period=period)
        else:
            thresh = SpikeGeneratorThreshold(N, spiketimes, period=period, sort=sort)
        
        if not hasattr(self, '_initialized'):
            NeuronGroup.__init__(self, N, model=LazyStateUpdater(), threshold=thresh, clock=clock)
            self._initialized = True
        else:
            self._threshold = thresh
 
    def reinit(self):
        super(SpikeGeneratorGroup, self).reinit()
        self._threshold.reinit()

    @property
    def spiketimes(self):
        return self._threshold.spiketimes
    
    @spiketimes.setter
    def spiketimes(self, values):
        self.__init__(self.N, values, period = self.period)

class FastSpikeGeneratorThreshold(Threshold):
    '''
    A faster version of the SpikeGeneratorThreshold where spikes are processed prior to the run (offline). It replaces the SpikeGeneratorThreshold as of 1.3.1.
    '''
    ## Notes:
    #  - N is ignored (should it not?)
    def __init__(self, N, addr, timestamps, dt = None, period=None):
        self.set_offsets(addr, timestamps, dt = dt)
        self.period = period
        self.dt = dt
        self.reinit()
        
    def set_offsets(self, I, T, dt = 1000):
        # Convert times into integers
        T = array(ceil(T/dt), dtype=int)
        # Put them into order
        # We use a field array to sort first by time and then by neuron index
        spikes = zeros(len(I), dtype=[('t', int), ('i', int)])
        spikes['t'] = T
        spikes['i'] = I
        spikes.sort(order=('t', 'i'))
        T = ascontiguousarray(spikes['t'])
        self.I = ascontiguousarray(spikes['i'])
        # Now for each timestep, we find the corresponding segment of I with
        # the spike indices for that timestep.
        # The idea of offsets is that the segment offsets[t]:offsets[t+1]
        # should give the spikes with time t, i.e. T[offsets[t]:offsets[t+1]]
        # should all be equal to t, and so then later we can return
        # I[offsets[t]:offsets[t+1]] at time t. It might take a bit of thinking
        # to see why this works. Since T is sorted, and bincount[i] returns the
        # number of elements of T equal to i, then j=cumsum(bincount(T))[t]
        # gives the first index in T where T[j]=t.
        if len(T):
            self.offsets = hstack((0, cumsum(bincount(T))))
        else:
            self.offsets = array([])
    
    def __call__(self, P):
        t = P.clock.t
        if self.period is not None:
            cp = int(t / self.period)
            if cp > self.curperiod:
                self.reinit()
                self.curperiod = cp
            t = t - cp * self.period
        dt = P.clock.dt
        t = int(round(t/dt))
        if t+1>=len(self.offsets):
            return array([], dtype=int)
        return self.I[self.offsets[t]:self.offsets[t+1]]
    
    def reinit(self):
        self.curperiod = -1
        
    @property
    def spiketimes(self):
        # this is a pain to do! retrieve spike times from offsets
        res = []
        for k in range(len(self.offsets)-1):
            idx = self.I[self.offsets[k]:self.offsets[k+1]]
            ts = [k*self.dt]*len(idx)
            res += zip(idx, ts)
        return res

class SpikeGeneratorThreshold(Threshold):
    """
    Old threshold object for the SpikeGeneratorGroup
    
    **Notes**

    This version of the SpikeGeneratorThreshold object is deprecated, since version 1.3.1 of Brian it has been replaced in most cases by the FastSpikeGeneratorThreshold. 
    This is kept only as a fallback object for when a SpikeGeneratorGroup object is initialized with a generator or an iterator object (see the doc for SpikeGeneratorGroup for more details). Please note that since this implementation is slower, using a static data structure as an input to a SpikeGeneratorGroup is advised.
    """
    def __init__(self, N, spiketimes, period=None, sort=True):
        self.set_spike_times(N, spiketimes, period=period, sort=sort)

    def reinit(self):
        def makeiter(obj):
            if callable(obj): return iter(obj())
            return iter(obj)
        self.spiketimeiter = makeiter(self.spiketimes)
        try:
            self.nextspikenumber, self.nextspiketime = self.spiketimeiter.next()
        except StopIteration:
            self.nextspiketime = None
            self.nextspikenumber = 0
        self.curperiod = -1

    def set_spike_times(self, N, spiketimes, period=None, sort=True):
        # N is the number of neurons, spiketimes is an iterable object of tuples (i,t) where
        # t is the spike time, and i is the neuron number. If spiketimes is a list or tuple,
        # then it will be sorted here.
        if isinstance(spiketimes, (list, tuple)) and sort:
            spiketimes = sorted(spiketimes, key=itemgetter(1))
        self.spiketimes = spiketimes
        self.N = N
        self.period = period
        self.reinit()
        
    def __call__(self, P):
        firing = zeros(self.N)
        t = P.clock.t
        if self.period is not None:
            cp = int(t / self.period)
            if cp > self.curperiod:
                self.reinit()
                self.curperiod = cp
            t = t - cp * self.period
        if isinstance(self.spiketimes, numpy.ndarray):
            t = float(t)
        while self.nextspiketime is not None and is_approx_less_than_or_equal(self.nextspiketime, t):
            if type(self.nextspikenumber)==int and firing[self.nextspikenumber]:
                log_warn('brian.SpikeGeneratorThreshold', 'Discarding multiple overlapping spikes')
            firing[self.nextspikenumber] = 1
            try:
                self.nextspikenumber, self.nextspiketime = self.spiketimeiter.next()
            except StopIteration:
                self.nextspiketime = None
        return where(firing)[0]

# The output of this function is fed into SpikeGeneratorGroup, consisting of
# time sorted pairs (t,i) where t is when neuron i fires
@check_units(t=second, n=1, sigma=second)
def PulsePacketGenerator(t, n, sigma):
    times = [pyrandom.gauss(t, sigma) for i in range(n)]
    times.sort()
    neuron = range(n)
    pyrandom.shuffle(neuron)
    return zip(neuron, times)


class PulsePacket(SpikeGeneratorGroup):
    """
    Fires a Gaussian distributed packet of n spikes with given spread
    
    **Initialised as:** :: 
    
        PulsePacket(t,n,sigma[,clock])
        
    with arguments:
    
    ``t``
        The mean firing time
    ``n``
        The number of spikes in the packet
    ``sigma``
        The standard deviation of the firing times.
    ``clock``
        The clock to use (omit to use default or local clock)
    
    **Methods**
    
    This class is derived from :class:`SpikeGeneratorGroup` and has all its
    methods as well as one additional method:
    
    .. method:: generate(t,n,sigma)
    
        Change the parameters and/or generate a new pulse packet.
    """
    @check_units(t=second, n=1, sigma=second)
    def __init__(self, t, n, sigma, clock=None):
        self.clock = guess_clock(clock)
        self.generate(t, n, sigma)

    def reinit(self):
        super(PulsePacket, self).reinit()
        self._threshold.reinit()
    @check_units(t=second, n=1, sigma=second)
    def generate(self, t, n, sigma):
        SpikeGeneratorGroup.__init__(self, n, PulsePacketGenerator(t, n, sigma), self.clock)

    def __repr__(self):
        return "Pulse packet neuron group"

class PoissonGroup(NeuronGroup):
    '''
    A group that generates independent Poisson spike trains.
    
    **Initialised as:** ::
    
        PoissonGroup(N,rates[,clock])
    
    with arguments:
    
    ``N``
        The number of neurons in the group
    ``rates``
        A scalar, array or function returning a scalar or array.
        The array should have the same length as the number of
        neurons in the group. The function should take one
        argument ``t`` the current simulation time.
    ``clock``
        The clock which the group will update with, do not
        specify to use the default clock.
    '''
    def __init__(self, N, rates=0 * hertz, clock=None):
        '''
        Initializes the group.
        P.rates gives the rates.
        '''
        NeuronGroup.__init__(self, N, model=LazyStateUpdater(), threshold=PoissonThreshold(),
                             clock=clock)
        if callable(rates): # a function is passed
            self._variable_rate = True
            self.rates = rates
            self._S0[0] = self.rates(self.clock.t)
        else:
            self._variable_rate = False
            self._S[0, :] = rates
            self._S0[0] = rates
        self.var_index = {'rate':0}

    def update(self):
        if self._variable_rate:
            self._S[0, :] = self.rates(self.clock.t)
        NeuronGroup.update(self)


class OfflinePoissonGroup(object): # This is weird, there is only an init method
    def __init__(self, N, rates, T):
        """
        Generates a Poisson group with N spike trains and given rates over the
        time window [0,T].
        """
        if isscalar(rates):
            rates = rates * ones(N)
        totalrate = sum(rates)
        isi = exponential(1 / totalrate, T * totalrate * 2)
        spikes = cumsum(isi)
        spikes = spikes[spikes <= T]
        neurons = randint(0, N, len(spikes))
        self.spiketimes = zip(neurons, spikes)


# Used in PoissonInput below
class EmptyGroup(object):
    def __init__(self, clock):
        self.clock = clock
    def get_spikes(self, delay):
        return None


class PoissonInput(Connection):
    """
    Adds a Poisson input to a NeuronGroup. Allows to efficiently simulate a large number of 
    independent Poisson inputs to a NeuronGroup variable, without simulating every synapse
    individually. The synaptic events are generated randomly during the simulation and
    are not preloaded and stored in memory (unless record=True is used).
    All the inputs must target the same variable, have the same frequency and same synaptic
    weight. You can use as many PoissonInput objects as you want, even targetting a same NeuronGroup.
    There is the possibility to consider time jitter in the presynaptic spikes, and
    synaptic unreliability. The inputs can also be recorded if needed. Finally, all
    neurons from the NeuronGroup receive independent realizations of Poisson spike trains,
    except if the keyword freeze=True is used, in which case all neurons receive the same
    Poisson input.
    
    **Initialised as:** ::
    
        PoissonInput(target[, N][, rate][, weight][, state][, jitter][, reliability][, copies][, record][, freeze])
    
    with arguments:
    
    ``target``
        The target :class:`NeuronGroup`
    ``N``
        The number of independent Poisson inputs
    ``rate``
        The rate of each Poisson process
    ``weight``
        The synaptic weight
    ``state``
        The name or the index of the synaptic variable of the :class:`NeuronGroup`      
    ``jitter``
        is ``None`` by default. There is the possibility to consider ``copies`` presynaptic
        spikes at each Poisson event, randomly shifted according to an exponential law
        with parameter ``jitter=taujitter`` (in second).
    ``reliability`` 
        is ``None`` by default. There is the possibility to consider ``copies`` presynaptic
        spikes at each Poisson event, where each of these spikes is unreliable, i.e. it occurs
        with probability ``jitter=alpha`` (between 0 and 1).
    ``copies``
        The number of copies of each Poisson event. This is identical to ``weight=copies*w``, except
        if ``jitter`` or ``reliability`` are specified.
    ``record``
        ``True`` if the input has to be recorded. In this case, the recorded events are
        stored in the ``recorded_events`` attribute, as a list of pairs ``(i,t)`` where ``i`` is the
        neuron index and ``t`` is the event time.
    ``freeze``
        ``True`` if the input must be the same for all neurons of the :class:`NeuronGroup`
    """    
    _record = []
    
    def __init__(self, target, N=None, rate=None, weight=None, state=None,
                  jitter=None, reliability=None, copies=1,
                  record=False, freeze=False):
        self.source = EmptyGroup(target.clock)
        self.target = target
        self.N = len(self.target)
        self.clock = target.clock
        self.delay = None
        self.iscompressed = True
        self.delays = None # delay to wait for the j-th synchronous spike to occur after the last sync event, for target neuron i
        self.lastevent = -inf * ones(self.N) # time of the last event for target neuron i
        self.events = []
        self.recorded_events = []

        self.n = N
        self.rate = rate
        self.w = weight
        self.var = state
        self._jitter = jitter
        
        if jitter is not None:
            self.delays = zeros((copies, self.N))
        
        self.reliability = reliability
        self.copies = copies
        self.record = record
        self.frozen = freeze
        
        if (jitter is not None) and (reliability is not None):
            raise Exception("Specifying both jitter and reliability is currently not supported.")

        if isinstance(state, str): # named state variable
            self.index = self.target.get_var_index(state)
        else:
            self.index = state

    @property
    def jitter(self):
        return self._jitter

    @jitter.setter
    def jitter(self, value):
        self._jitter = value
        if value is not None:
            self.delays = zeros((self.copies, self.N))

    def propagate(self, spikes):
        i = 0
        
        n = self.n
        f = self.rate
        w = self.w
        var = self.var
        jitter = self.jitter
        reliability = self.reliability
        record = self.record
        frozen = self.frozen
        state = self.index
        
        if (jitter==None) and (reliability==None):
            if frozen:
                rnd = binomial(n=n, p=f * self.clock.dt)
                self.target._S[state, :] += w * rnd
                if rnd > 0:
                    self.events.append(self.clock.t)
            else:
                rnd = binomial(n=n, p=f * self.clock.dt, size=(self.N))
                self.target._S[state, :] += w * rnd
                ind = nonzero(rnd>0)[0]
                if record and len(ind)>0:
                    self.recorded_events.append((ind[0], self.clock.t))
        elif (jitter is not None):
            p = self.copies
            taujitter = jitter
            if (p > 0) & (f > 0):
                k = binomial(n=n, p=f * self.clock.dt, size=(self.N)) # number of synchronous events here, for every target neuron
                syncneurons = (k > 0) # neurons with a syncronous event here
                self.lastevent[syncneurons] = self.clock.t
                if taujitter == 0.0:
                    self.delays[:, syncneurons] = zeros((p, sum(syncneurons)))
                else:
                    self.delays[:, syncneurons] = exponential(scale=taujitter, size=(p, sum(syncneurons)))
                # Delayed spikes occur now
                lastevent = tile(self.lastevent, (p, 1))
                b = (abs(self.clock.t - (lastevent + self.delays)) <= (self.clock.dt / 2) * ones((p, self.N))) # delayed spikes occurring now
                weff = sum(b, axis=0) * w
                self.target._S[state, :] += weff
        elif (reliability is not None):
            p = self.copies
            alpha = reliability
            if (p > 0) & (alpha > 0):
                weff = w * binomial(n=p, p=alpha)
                self.target._S[state, :] += weff * binomial(n=n, p=f * self.clock.dt, size=(self.N))

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



from brian import *


class FutureSpikeContainer(object):
    def __init__(self, nsteps):
        self.nsteps = nsteps
        self.maxspikes = 2
        self.ind = zeros((2, nsteps), dtype=int)
        self.nspikes = zeros(nsteps, dtype=int)
        self.cursor = 0

    def insert(self, spikes, delays):
        t = (self.timecursor + delays) % self.nsteps
        I = argsort(t)
        t = t[I]
        spikes = spikes[I]
        # TODO: finish this - use scheduled_events.py which basically does the
        # same thing

        # assume for the moment that no overflows happen

    def getspikes(self):
        spikes = self.ind[0:self.nspikes[self.cursor], self.cursor]
        self.nspikes[timecursor] = 0
        self.timecursor += 1
        if self.timecursor == self.nsteps:
            self.timecursor = 0
        return spikes

if __name__ == '__pass__':
    pass



'''
Tutorial 1a: The simplest Brian program
***************************************

Importing the Brian module
~~~~~~~~~~~~~~~~~~~~~~~~~~

The first thing to do in any Brian program is to load Brian and the names of
its functions and classes. The standard way to do this is to use the Python
``from ... import *`` statement. 
'''
from brian import *
'''
Integrate and Fire model
~~~~~~~~~~~~~~~~~~~~~~~~

The neuron model we will use in this tutorial is the simplest possible
leaky integrate and fire neuron, defined by the differential equation:

    tau dV/dt = -(V-El)

and with a threshold value Vt and reset value Vr.

Parameters
~~~~~~~~~~

Brian has a system for defining physical quantities (quantities with
a physical dimension such as time). The code below illustrates how
to use this system, which (mostly) works just as you'd expect.
'''
tau = 20 * msecond        # membrane time constant
Vt = -50 * mvolt          # spike threshold
Vr = -60 * mvolt          # reset value
El = -60 * mvolt          # resting potential (same as the reset)
'''
The built in standard units in Brian consist of all the fundamental
SI units like second and metre, along with a selection of derived
SI units such as volt, farad, coulomb. All names are lowercase
following the SI standard. In addition, there are scaled versions
of these units using the standard SI prefixes m=1/1000, K=1000, etc. 

Neuron model and equations
~~~~~~~~~~~~~~~~~~~~~~~~~~

The simplest way to define a neuron model in Brian is to write a list
of the differential equations that define it. For the moment, we'll just
give the simplest possible example, a single differential equation. You
write it in the following form::

    dx/dt = f(x) : unit

where ``x`` is the name of the variable, ``f(x)`` can be any valid Python
expression, and ``unit`` is the physical units of the variable ``x``. In our
case we will write::

    dV/dt = -(V-El)/tau : volt

to define the variable ``V`` with units ``volt``.

To complete the specification of the model, we also define a threshold and reset
value and create a group of 40 neurons with this model.
'''
G = NeuronGroup(N=40, model='dV/dt = -(V-El)/tau : volt',
              threshold=Vt, reset=Vr)
'''
The statement creates a new object 'G' which is an instance of the
Brian class :class:`NeuronGroup`, initialised with the values in the
line above and 40 neurons. In Python, you can call a function or initialise
a class using keyword arguments as well as ordered arguments, so
if I defined a function ``f(x,y)`` I could call it as ``f(1,2)`` or
as ``f(y=2,x=1)`` and get the same effect. See the Python tutorial
for more information on this.

For the moment, we leave the neurons in this group unconnected
to each other, each evolves separately from the others.

Simulation
~~~~~~~~~~

Finally, we run the simulation for 1 second of simulated time.
By default, the simulator uses a timestep dt = 0.1 ms.
'''
run(1 * second)
'''
And that's it! To see some of the output of this network, go
to the next part of the tutorial.

Exercise
~~~~~~~~

The units system of Brian is useful for ensuring that everything
is consistent, and that you don't make hard to find mistakes in
your code by using the wrong units. Try changing the units of one
of the parameters and see what happens.

Solution
~~~~~~~~

You should see an error message with a Python traceback (telling
you which functions were being called when the error happened),
ending in a line something like::

    Brian.units.DimensionMismatchError: The differential equations
    are not homogeneous!, dimensions were (m^2 kg s^-3 A^-1)
    (m^2 kg s^-4 A^-1) 
'''



'''
This example demonstrates using different clocks for different objects
in the network. The clock ``simclock`` is the clock used for the
underlying simulation. The clock ``monclock`` is the clock used for
monitoring the membrane potential. This monitoring takes place less
frequently than the simulation update step to save time and memory.
Finally, the clock ``inputclock`` controls when the external 'current'
``Iext`` should be updated. In this  case, we update it infrequently
so we can see the effect on the network.

This example also demonstrates the @network_operation decorator. A
function with this decorator will be run as part of the network
update step, in sync with the clock provided (or the default one
if none is provided).
'''

from brian import *
# define the three clocks
simclock = Clock(dt=0.1 * ms)
monclock = Clock(dt=0.3 * ms)
inputclock = Clock(dt=100 * ms)
# simple leaky I&F model with external 'current' Iext as a parameter
tau = 10 * ms
eqs = '''
dV/dt = (-V+Iext)/tau : volt
Iext: volt
'''
# A single leaky I&F neuron with simclock as its clock
G = NeuronGroup(1, model=eqs, reset=0 * mV, threshold=10 * mV, clock=simclock)
G.V = 5 * mV
# This function will be run in sync with inputclock i.e. every 100 ms
@network_operation(clock=inputclock)
def update_Iext():
    G.Iext = rand(len(G)) * 20 * mV
# V is monitored in sync with monclock
MV = StateMonitor(G, 'V', record=0, clock=monclock)
# run and plot 
run(1000 * ms)
plot(MV.times / ms, MV[0] / mV)
show()
# You should see 10 different regions, sometimes Iext will be above threshold
# in which case you will see regular spiking at different rates, and sometimes
# it will be below threshold in which case you'll see exponential decay to that
# value



from brian import *
from brian.experimental.codegen import *
eqs = Equations('''
dV/dt = -W*V/(10*second) : volt 
dW/dt = -V**2/(1*second) : volt
''')
scheme = exp_euler_scheme
print 'Equations'
print '========='
print eqs
print 'Scheme'
print '======'
for block_specifier, block_code in scheme:
    print block_specifier
    print block_code
print 'Python code'
print '==========='
print PythonCodeGenerator().generate(eqs, scheme)
print 'C code'
print '======'
print CCodeGenerator().generate(eqs, scheme)
print 'GPU code'
print '======'
print GPUCodeGenerator().generate(eqs, scheme)
